@@ -21,6 +21,7 @@
 #include "iconset.h"
 #include "zip.h"
 
+#include <qobject.h>
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qtimer.h>
@@ -575,6 +576,17 @@ void Icon::removeAnim(bool doDetach)
 	//emit d->iconModified( d->impix.pixmap() );
 }
 
+//!
+//! Returns the number of current animation frame.
+//! \sa setAnim()
+int Icon::framenumber() const
+{
+	if ( d->anim )
+		return d->anim->framenumber();
+
+	return 0;
+}
+
 //!
 //! Returns name of the Icon.
 //! \sa setName()
@@ -667,8 +679,8 @@ bool Icon::loadFromData(const QByteArray &ba, bool isAnim)
 	if ( isAnim ) {
 		Anim *anim = new Anim(ba);
 		setAnim(*anim);
+		ret = anim->numFrames();
 		delete anim; // shared data rules ;)
-		ret = true;
 	}
 
 	if ( !ret && d->impix.loadFromData(ba) )
@@ -760,6 +772,7 @@ class IconsetFactoryPrivate
 {
 private:
 	static QPtrList<Iconset> *iconsets;
+	static QPixmap *emptyPixmap;
 
 public:
 	static void registerIconset(const Iconset *);
@@ -773,6 +786,7 @@ public:
 //! \endif
 
 QPtrList<Iconset> *IconsetFactoryPrivate::iconsets = 0;
+QPixmap *IconsetFactoryPrivate::emptyPixmap = 0;
 
 void IconsetFactoryPrivate::registerIconset(const Iconset *i)
 {
@@ -790,11 +804,18 @@ void IconsetFactoryPrivate::unregisterIconset(const Iconset *i)
 	if ( iconsets && iconsets->find(i) >= 0 ) {
 		//qWarning("IconsetFactoryPrivate::unregisterIconset(): %s", i->fileName().latin1());
 		iconsets->remove ( i );
+	}
 
-		if ( !iconsets->count() ) {
+	if ( !iconsets || !iconsets->count() ) {
+		if ( iconsets ) {
 			delete iconsets;
 			iconsets = 0;
 		}
+
+		if ( emptyPixmap ) {
+			delete emptyPixmap;
+			emptyPixmap = 0;
+		}
 	}
 }
 
@@ -839,6 +860,22 @@ Icon IconsetFactory::icon(const QString &name)
 	return Icon();
 }
 
+//!
+//! Returns QPixmap of first animation frame of the specified Icon, or the empty
+//! QPixmap, if that Icon wasn't found in IconsetFactory.
+//! This function is faster than the call to IconsetFactory::icon() and cast to QPixmap,
+//! because the intermediate Icon object is not created and destroyed.
+const QPixmap &IconsetFactory::iconPixmap(const QString &name)
+{
+	const Icon *i = iconPtr(name);
+	if ( i )
+		return i->impix().pixmap();
+
+	if ( !IconsetFactoryPrivate::emptyPixmap )
+		IconsetFactoryPrivate::emptyPixmap = new QPixmap();
+	return *IconsetFactoryPrivate::emptyPixmap;
+}
+
 //!
 //! Returns list of all Icon names that are in IconsetFactory.
 const QStringList IconsetFactory::icons()
@@ -1010,7 +1047,8 @@ public:
 
 	static int icon_counter; // used to give unique names to icons
 
-	void loadIcon(const QDomElement &i, const QString &dir)
+	// will return 'true' when icon is loaded ok
+	bool loadIcon(const QDomElement &i, const QString &dir)
 	{
 		Icon icon;
 
@@ -1183,14 +1221,19 @@ public:
 
 		if ( loadSuccess )
 			append( name, new Icon(icon) );
+
+		return loadSuccess;
 	}
 
+	// would return 'true' on success
 	bool load(const QDomDocument &doc, const QString dir)
 	{
 		QDomElement base = doc.documentElement();
 		if ( base.tagName() != "icondef" )
 			return false;
 
+		bool success = true;
+
 		for(QDomNode node = base.firstChild(); !node.isNull(); node = node.nextSibling()) {
 			QDomElement i = node.toElement();
 			if( i.isNull() )
@@ -1201,14 +1244,16 @@ public:
 				loadMeta (i, dir);
 			}
 			else if ( tag == "icon" ) {
-				loadIcon (i, dir);
+				bool ret = loadIcon (i, dir);
+				if ( !ret )
+					success = false;
 			}
 			else if ( tag == "x" ) {
 				info.insert( i.attribute("xmlns"), new QString(i.text()) );
 			}
 		}
 
-		return true;
+		return success;
 	}
 };
 //! \endif
