@@ -318,19 +318,29 @@ public:
 		setText(2, pa->isActive() ? AccountManageDlg::tr("Active") : AccountManageDlg::tr("Not active"));
 		setOn(pa->enabled());
 	}
-	
+
 	void stateChange( bool s)
 	{
-
-		if(s || !pa->isActive() && !pa->eventQueue()->count()){
-			if(pa->enabled()!=s) pa->setEnabled(s);
-		}else{
-			setOn(!s);
-			if(pa->isActive())
-				QMessageBox::information(listView(), AccountManageDlg::tr("Error"), AccountManageDlg::tr("Unable to disable the account, as it is currently active."));
-			else
+		if (!s) {
+			if (pa->eventQueue()->count()) {
+				setOn(!s);
 				QMessageBox::information(listView(), AccountManageDlg::tr("Error"), AccountManageDlg::tr("Unable to disable the account, as it has pending events."));
+				return;
+			}
+
+			if (pa->isActive()) {
+				if (QMessageBox::information(listView(), AccountManageDlg::tr("Disable Account"), AccountManageDlg::tr("The account is currently active.\nDo you want to log out ?"),QMessageBox::Yes,QMessageBox::No | QMessageBox::Default | QMessageBox::Escape, QMessageBox::NoButton) == QMessageBox::Yes) {
+					pa->logout();
+				}
+				else {
+					setOn(!s);
+					return;
+				}
+			}
 		}
+
+		if (pa->enabled()!=s)
+			pa->setEnabled(s);
 	}
 
 	int rtti() const
