@@ -34,6 +34,10 @@
 #include"busywidget.h"
 #include"common.h"
 #include"iconwidget.h"
+#include"tasklist.h"
+#include"xmpp_vcard.h"
+#include"vcardfactory.h"
+#include"infodlg.h"
 
 class AddUserDlg::Private
 {
@@ -44,6 +48,7 @@ public:
 	BusyWidget *busy;
 	QStringList services;
 	JT_Gateway *jt;
+	TaskList *tasks;
 };
 
 AddUserDlg::AddUserDlg(const QStringList &services, const QStringList &names, const QStringList &groups, PsiAccount *pa)
@@ -55,6 +60,9 @@ AddUserDlg::AddUserDlg(const QStringList &services, const QStringList &names, co
 	connect(d->pa, SIGNAL(updatedActivity()), SLOT(pa_updatedActivity()));
 	d->services = services;
 	d->jt = 0;
+	d->tasks = new TaskList;
+	connect(d->tasks, SIGNAL(started()),  busy, SLOT(start()));
+	connect(d->tasks, SIGNAL(finished()), busy, SLOT(stop()));
 
 	setCaption(CAP(caption()));
 	setIcon(IconsetFactory::icon("psi/addContact"));
@@ -83,6 +91,9 @@ AddUserDlg::AddUserDlg(const QStringList &services, const QStringList &names, co
 	connect(pb_close, SIGNAL(clicked()), SLOT(reject()));
 	connect(pb_transGet, SIGNAL(clicked()), SLOT(getTransID()));
 
+	connect(tb_vCard, SIGNAL(clicked()), SLOT(getVCardActivated()));
+	connect(tb_resolveNick, SIGNAL(clicked()), SLOT(resolveNickActivated()));
+
 	ck_authreq->setChecked(true);
 	ck_close->setChecked(true);
 
@@ -91,7 +102,7 @@ AddUserDlg::AddUserDlg(const QStringList &services, const QStringList &names, co
 
 AddUserDlg::~AddUserDlg()
 {
-	delete d->jt;
+	delete d->tasks;
 	d->pa->dialogUnregister(this);
 	delete d;
 }
@@ -134,7 +145,7 @@ void AddUserDlg::ok()
 
 void AddUserDlg::serviceActivated(int x)
 {
-	if(d->busy->isActive()) {
+	if(d->jt) {
 		delete d->jt;
 		d->jt = 0;
 		d->busy->stop();
@@ -152,7 +163,7 @@ void AddUserDlg::serviceActivated(int x)
 		connect(d->jt, SIGNAL(finished()), SLOT(jt_getFinished()));
 		d->jt->get(Jid(d->services[x]));
 		d->jt->go(true);
-		d->busy->start();
+		d->tasks->append( d->jt );
 	}
 }
 
@@ -171,12 +182,11 @@ void AddUserDlg::getTransID()
 	connect(d->jt, SIGNAL(finished()), SLOT(jt_setFinished()));
 	d->jt->set(Jid(d->services[cb_service->currentItem()-1]), le_transPrompt->text());
 	d->jt->go(true);
-	d->busy->start();
+	d->tasks->append( d->jt );
 }
 
 void AddUserDlg::jt_getFinished()
 {
-	d->busy->stop();
 	JT_Gateway *jt = d->jt;
 	d->jt = 0;
 
@@ -191,7 +201,6 @@ void AddUserDlg::jt_getFinished()
 
 void AddUserDlg::jt_setFinished()
 {
-	d->busy->stop();
 	cb_service->setEnabled(true);
 	le_transPrompt->setEnabled(true);
 	pb_transGet->setEnabled(true);
@@ -233,3 +242,38 @@ void AddUserDlg::errorGateway(const QString &str, const QString &err)
 		"</qt>"
 		).arg(str).arg(err));
 }
+
+void AddUserDlg::getVCardActivated()
+{
+	Jid j(le_jid->text().stripWhiteSpace());
+
+	const VCard *vcard = VCardFactory::vcard(j);
+
+	VCard tmp;
+	if ( vcard )
+		tmp = *vcard;
+
+	InfoDlg *w = new InfoDlg(InfoDlg::Contact, j, tmp, d->pa, 0, 0, false);
+	w->show();
+
+	// automatically retrieve info if it doesn't exist
+	if(!vcard)
+		w->doRefresh();
+}
+
+void AddUserDlg::resolveNickActivated()
+{
+	Jid j(le_jid->text().stripWhiteSpace());
+	JT_VCard *jt = VCardFactory::getVCard(j, d->pa->client()->rootTask(), this, SLOT(resolveNickFinished()), false);
+	d->tasks->append( jt );
+}
+
+void AddUserDlg::resolveNickFinished()
+{
+	JT_VCard *jt = (JT_VCard *)sender();
+
+	if(jt->success()) {
+		if ( !jt->vcard().nickName().isEmpty() )
+			le_nick->setText( jt->vcard().nickName() );
+	}
+}
