@@ -49,6 +49,10 @@ private slots:
 	void jt_setFinished();
 	void le_transPromptChanged(const QString &);
 
+	void getVCardActivated();
+	void resolveNickActivated();
+	void resolveNickFinished();
+
 private:
 	class Private;
 	Private *d;
