@@ -35,6 +35,8 @@ public:
 	const QImage &image() const;
 	const QIconSet & iconSet() const;
 	const Impix &impix() const;
+	int framenumber() const;
+	const QString &name() const;
 
 public slots:
 	void activated(bool playSound = true);
