@@ -22,6 +22,7 @@
 
 #include<qlabel.h>
 #include<qcursor.h>
+#include<qdragobject.h>
 #include<qlineedit.h>
 #include<qtoolbutton.h>
 #include<qlayout.h>
@@ -39,11 +40,28 @@
 #include"msgmle.h"
 #include"iconselect.h"
 #include"psicon.h"
+#include"psitoolbar.h"
+#include"iconaction.h"
+//#include"avatars.h"
 
 #ifdef Q_WS_WIN
 #include<windows.h>
 #endif
 
+//----------------------------------------------------------------------------
+// Some required actions
+//----------------------------------------------------------------------------
+
+class StretchWidget : public QWidget
+{
+public:
+	StretchWidget(QWidget *parent)
+	: QWidget(parent)
+	{
+		setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+	}
+};
+
 //----------------------------------------------------------------------------
 // ChatDlg
 //----------------------------------------------------------------------------
@@ -65,13 +83,18 @@ public:
 	ChatView *log;
 	ChatEdit *mle;
 
+	// Avatars
+	//QLabel *avatar;
+
 	QLabel *lb_ident;
 	IconLabel *lb_status;
 	QLineEdit *le_jid;
 	QLCDNumber *lcd_count;
-	IconToolButton *tb_send, *tb_clear, *tb_history, *tb_info, *tb_pgp, *tb_icon, *tb_file;
 	QPopupMenu *pm_settings;
 
+	PsiToolBar *toolbar;
+	IconAction *act_send, *act_clear, *act_history, *act_info, *act_pgp, *act_icon, *act_file;
+
 	int pending;
 	bool keepOpen, warnSend;
 
@@ -85,6 +108,17 @@ public:
 	bool lastWasEncrypted;
 	bool smallChat;
 
+	// Message Events
+	QTimer *composingTimer;
+	bool isComposing;
+	bool contactIsComposing;
+	bool sendComposingEvents;
+	QString eventId;
+
+signals:
+	// Signals if user (re)started/stopped composing
+	void composing(bool);
+
 public slots:
 	void addEmoticon(const Icon *icon) {
 		if ( !dlg->isActiveWindow() )
@@ -115,6 +149,29 @@ public slots:
 	void updateCounter() {
 		lcd_count->display((int)mle->text().length());
 	}
+
+	// Records that the user is composing
+	void setComposing() {
+		if (!composingTimer) {
+			/* User (re)starts composing */
+			composingTimer = new QTimer(this);
+			connect(composingTimer, SIGNAL(timeout()), SLOT(checkComposing()));
+			composingTimer->start(2000); // FIXME: magic number
+			emit composing(true);
+		}
+		isComposing = true;
+	}
+
+	// Checks if the user is still composing
+	void checkComposing() {
+		if (!isComposing) {
+			// User stopped composing
+			delete composingTimer;
+			composingTimer = 0;
+			emit composing(false);
+		}
+		isComposing = false; // Reset composing
+	}
 };
 
 ChatDlg::ChatDlg(const Jid &jid, PsiAccount *pa)
@@ -133,6 +190,8 @@ ChatDlg::ChatDlg(const Jid &jid, PsiAccount *pa)
 	d->key = "";
 	d->lastWasEncrypted = false;
 
+	setAcceptDrops(TRUE);
+
 	QVBoxLayout *vb1 = new QVBoxLayout(this, 4);
 	QSplitter *sp = new QSplitter(Vertical, this);
 	vb1->addWidget(sp);
@@ -166,45 +225,56 @@ ChatDlg::ChatDlg(const Jid &jid, PsiAccount *pa)
 	// tool area
 	QVBoxLayout *vb3 = new QVBoxLayout(sp_bottom, 4);
 	QHBoxLayout *hb3 = new QHBoxLayout(vb3);
-	d->tb_clear = new IconToolButton(sp_bottom);
-	connect(d->tb_clear, SIGNAL(clicked()), SLOT(doClearButton()));
-	QToolTip::add(d->tb_clear, tr("Clear chat window"));
-	hb3->addWidget(d->tb_clear);
-	hb3->addStretch(1);
 
-	// icon selector
-	//connect(pa->psi()->iconSelectPopup(), SIGNAL(iconSelected(const Icon *)), d, SLOT(addEmoticon(const Icon *)));
+	d->toolbar = new PsiToolBar(tr("Chat toolbar"), 0, sp_bottom);
+	d->toolbar->setCustomizeable( false ); // it isn't ready now, and we don't want segfaults
+	d->toolbar->setFrameShape( QFrame::NoFrame );
+	hb3->addWidget( d->toolbar );
+
+	d->act_clear = new IconAction (tr("Clear chat window"), "psi/clearChat", tr("Clear chat window"), 0, this);
+	connect( d->act_clear, SIGNAL( activated() ), SLOT( doClearButton() ) );
+
 	connect(pa->psi()->iconSelectPopup(), SIGNAL(textSelected(QString)), d, SLOT(addEmoticon(QString)));
-	d->tb_icon = new IconToolButton(sp_bottom);
-	d->tb_icon->setPopup(pa->psi()->iconSelectPopup());
-	d->tb_icon->setPopupDelay (1);
-	QToolTip::add(d->tb_icon, tr("Select icon"));
-	hb3->addWidget(d->tb_icon);
+	d->act_icon = new IconAction( tr( "Select icon" ), "psi/smile", tr( "Select icon" ), 0, this );
+	d->act_icon->setPopup( pa->psi()->iconSelectPopup() );
 
-	d->tb_file = new IconToolButton(sp_bottom);
-	connect(d->tb_file, SIGNAL(clicked()), SLOT(doFile()));
-	QToolTip::add(d->tb_file, tr("Send file"));
-	hb3->addWidget(d->tb_file);
+	d->act_file = new IconAction( tr( "Send file" ), "psi/upload", tr( "Send file" ), 0, this );
+	connect( d->act_file, SIGNAL( activated() ), SLOT( doFile() ) );
 
-	d->tb_pgp = new IconToolButton(sp_bottom);
-	d->tb_pgp->setToggleButton(true);
-	QToolTip::add(d->tb_pgp, tr("Toggle encryption"));
-	hb3->addWidget(d->tb_pgp);
+	d->act_pgp = new IconAction( tr( "Toggle encryption" ), "psi/cryptoNo", tr( "Toggle encryption" ), 0, this, 0, true );
 
-	d->tb_info = new IconToolButton(sp_bottom);
-	connect(d->tb_info, SIGNAL(clicked()), SLOT(doInfo()));
-	QToolTip::add(d->tb_info, tr("User info"));
-	hb3->addWidget(d->tb_info);
+	d->act_info = new IconAction( tr( "User info" ), "psi/vCard", tr( "User info" ), 0, this );
+	connect( d->act_info, SIGNAL( activated() ), SLOT( doInfo() ) );
 
-	d->tb_history = new IconToolButton(sp_bottom);
-	connect(d->tb_history, SIGNAL(clicked()), SLOT(doHistory()));
-	QToolTip::add(d->tb_history, tr("Message history"));
-	hb3->addWidget(d->tb_history);
+	d->act_history = new IconAction( tr( "Message history" ), "psi/history", tr( "Message history" ), 0, this );
+	connect( d->act_history, SIGNAL( activated() ), SLOT( doHistory() ) );
 
+	d->act_clear->addTo( d->toolbar );
+	d->toolbar->setStretchableWidget(new StretchWidget(d->toolbar));
+
+	d->act_icon->addTo( d->toolbar );
+	d->act_file->addTo( d->toolbar );
+	d->act_pgp->addTo( d->toolbar );
+	d->act_info->addTo( d->toolbar );
+	d->act_history->addTo( d->toolbar );
+
+	// Bottom row
+	QHBoxLayout *hb4 = new QHBoxLayout(vb3);
+
+	// Text input
 	d->mle = new ChatEdit(sp_bottom);
-	vb3->addWidget(d->mle);
+	hb4->addWidget(d->mle);
 	connect(d->mle, SIGNAL(textChanged()), d, SLOT(updateCounter()));
 
+	// Avatars
+	//d->avatar = new QLabel(sp_bottom);
+	//hb4->addWidget(d->avatar);
+	//updateAvatar();
+	// FIXME: Maybe the psiaccount should do this, to avoid a signal to each
+	// open window when an avatar changes.
+	//connect(d->pa->avatarFactory(),SIGNAL(avatarChanged(const Jid&)), this, SLOT(updateAvatar(const Jid&)));
+	//connect(d->pa->psi(),SIGNAL(emitOptionsUpdate()), this, SLOT(updateAvatar()));
+
 	d->pm_settings=new QPopupMenu(this);
 
 	connect(d->pm_settings, SIGNAL(aboutToShow()), SLOT(buildMenu()));
@@ -216,6 +286,14 @@ ChatDlg::ChatDlg(const Jid &jid, PsiAccount *pa)
 	d->status = -1;
 	d->pa->dialogRegister(this, d->jid);
 
+	// Message events
+	d->contactIsComposing = false;
+	d->sendComposingEvents = false;
+	d->isComposing = false;
+	d->composingTimer = 0;
+	connect(d->mle, SIGNAL(textChanged()), d, SLOT(setComposing()));
+	connect(d, SIGNAL(composing(bool)), SLOT(updateIsComposing(bool)));
+
 	updateContact(d->jid, true);
 
 	d->smallChat=option.smallChats;
@@ -234,17 +312,13 @@ ChatDlg::ChatDlg(const Jid &jid, PsiAccount *pa)
 
 	UserListItem *u = d->pa->findFirstRelavent(d->jid);
 	if(u && u->isSecure(d->jid.resource()))
-		d->tb_pgp->setOn(true);
+		d->act_pgp->setOn(true);
 }
 
 ChatDlg::~ChatDlg()
 {
 	d->pa->dialogUnregister(this);
 
-	//QMimeSourceFactory *m = d->log->mimeSourceFactory();
-	//d->log->setMimeSourceFactory(QMimeSourceFactory::defaultFactory());
-	//delete m;
-
 	delete d;
 }
 
@@ -301,6 +375,16 @@ void ChatDlg::closeEvent(QCloseEvent *e)
 			setSelfDestruct(60 * 24);
 	}
 
+	// Reset 'contact is composing' & cancel own composing event
+	updateContactIsComposing(false);
+	if (d->composingTimer) {
+		delete d->composingTimer;
+		d->composingTimer = 0;
+		d->isComposing = false;
+		updateIsComposing(false);
+	}
+
+
 	if(d->pending > 0) {
 		d->pending = 0;
 		messagesRead(d->jid);
@@ -339,6 +423,21 @@ void ChatDlg::windowActivationChange(bool oldstate)
 	}
 }
 
+
+void ChatDlg::dropEvent(QDropEvent* event)
+{
+	QStringList l;
+	if (d->pa->loggedIn() && QUriDrag::decodeLocalFiles(event,l) && !l.isEmpty())
+		d->pa->actionSendFiles(d->jid,l);
+}
+
+void ChatDlg::dragEnterEvent(QDragEnterEvent* event)
+{
+	QStringList l;
+	event->accept(d->pa->loggedIn() && QUriDrag::canDecode(event) && QUriDrag::decodeLocalFiles(event,l) && !l.isEmpty());
+}
+
+
 const Jid & ChatDlg::jid() const
 {
 	return d->jid;
@@ -407,12 +506,12 @@ void ChatDlg::updateContact(const Jid &jid, bool fromPresence)
 			d->statusString = statusString;
 		}
 
-		//if(statusChanged) {
+		if(statusChanged) {
 			if(status == -1 || !u)
 				d->lb_status->setIcon(IconsetFactory::iconPtr("status/noauth"));
 			else
 				d->lb_status->setIcon(is->statusPtr(jid, status));
-		//}
+		}
 
 		if(u)
 			QToolTip::add(d->lb_status, u->makeTip(true, false));
@@ -454,9 +553,52 @@ void ChatDlg::updateContact(const Jid &jid, bool fromPresence)
 				appendSysMsg(msg);
 			}
 		}
+
+		// Reset 'is composing' event if the status changed
+		if (statusChanged) {
+			updateContactIsComposing(false);
+		}
 	}
 }
 
+
+// Avatars
+//void ChatDlg::updateAvatar(const Jid& j)
+//{
+//	if (j.compare(d->jid,false))
+//		updateAvatar();
+//}
+
+//void ChatDlg::updateAvatar()
+//{
+//	QString res;
+//	QString client;
+
+//	if (!option.avatarsEnabled || !option.avatarsChatdlgEnabled) {
+//		d->avatar->hide();
+//		return;
+//	}
+//
+//	UserListItem *ul = d->pa->findFirstRelavent(d->jid);
+//	if (ul && !ul->userResourceList().isEmpty()) {
+//		UserResourceList::Iterator it = ul->userResourceList().find(d->jid.resource());
+//		if(it == ul->userResourceList().end())
+//			it = ul->userResourceList().priority();
+//
+//		res = (*it).name();
+//		client = (*it).clientName();
+//	}
+//	QPixmap p = d->pa->avatarFactory()->getAvatar(d->jid.withResource(res),client);
+//	if (p.isNull()) {
+//		d->avatar->hide();
+//	}
+//	else {
+//		d->avatar->setPixmap(p);
+//		d->avatar->show();
+//	}
+//}
+
+
 void ChatDlg::setLooks()
 {
 	// update the font
@@ -466,29 +608,16 @@ void ChatDlg::setLooks()
 	d->mle->setFont(f);
 
 	if (d->smallChat) {
-		//hide the first row
 		d->lb_ident->hide();
 		d->lb_status->hide();
 		d->le_jid->hide();
-		d->lcd_count->hide();
-		//hide the toolbar row
-		d->tb_clear->hide();
-		d->tb_file->hide();
-		d->tb_pgp->hide();
-		d->tb_info->hide();
-		d->tb_history->hide();
+		d->toolbar->hide();
 	}
 	else {
-		//show the first row
 		d->lb_ident->show();
 		d->lb_status->show();
 		d->le_jid->show();
-		//show the toolbar row
-		d->tb_clear->show();
-		d->tb_file->show();
-		d->tb_pgp->show();
-		d->tb_info->show();
-		d->tb_history->show();
+		d->toolbar->show();
 	}
 
 	if ( option.showCounter && !d->smallChat )
@@ -496,32 +625,30 @@ void ChatDlg::setLooks()
 	else
 		d->lcd_count->hide();
 
-	if ( option.useEmoticons&& !d->smallChat )
-		d->tb_icon->show();
-	else
-		d->tb_icon->hide();
-
 	// update contact info
 	d->status = -2; // sick way of making it redraw the status
 	updateContact(d->jid, false);
 
 	// toolbuttons
-	d->tb_clear->setIcon(IconsetFactory::iconPtr("psi/clearChat"));
-	d->tb_info->setIcon(IconsetFactory::iconPtr("psi/vCard"));
-	d->tb_history->setIcon(IconsetFactory::iconPtr("psi/history"));
-	d->tb_icon->setIcon(IconsetFactory::iconPtr("psi/smile"));
 	QIconSet i;
 	i.setPixmap(IconsetFactory::icon("psi/cryptoNo"),  QIconSet::Automatic, QIconSet::Normal, QIconSet::Off);
 	i.setPixmap(IconsetFactory::icon("psi/cryptoYes"), QIconSet::Automatic, QIconSet::Normal, QIconSet::On);
-	d->tb_pgp->setIconSet( i );
-	d->tb_file->setIcon(IconsetFactory::iconPtr("psi/upload"));
+	d->act_pgp->setIcon( 0 );
+	d->act_pgp->setIconSet( i );
 
 	// update the widget icon
+#ifndef Q_WS_MAC
 	setIcon(IconsetFactory::icon("psi/chat"));
+#endif
 }
 
 void ChatDlg::optionsUpdate()
 {
+	if (option.oldSmallChats!=option.smallChats)
+	{
+		d->smallChat=option.smallChats;
+	}
+	
 	setLooks();
 
 	if(isHidden()) {
@@ -543,11 +670,11 @@ void ChatDlg::optionsUpdate()
 void ChatDlg::updatePGP()
 {
 	if(!d->pa->pgpKey().isEmpty()) {
-		d->tb_pgp->setEnabled(true);
+		d->act_pgp->setEnabled(true);
 	}
 	else {
-		d->tb_pgp->setOn(false);
-		d->tb_pgp->setEnabled(false);
+		d->act_pgp->setOn(false);
+		d->act_pgp->setEnabled(false);
 	}
 }
 
@@ -617,6 +744,9 @@ void ChatDlg::updateCaption()
 	}
 	cap += d->dispNick;
 
+	if (d->contactIsComposing)
+		cap = tr("%1 (Composing ...)").arg(cap);
+
 	// if taskbar flash, then we need to erase first and redo
 #ifdef Q_WS_WIN
 	bool on = false;
@@ -700,11 +830,19 @@ void ChatDlg::doSend()
 	m.setType("chat");
 	m.setBody(d->mle->text());
 	m.setTimeStamp(QDateTime::currentDateTime());
-	if(d->tb_pgp->isOn())
+	if(d->act_pgp->isOn())
 		m.setWasEncrypted(true);
 	d->m = m;
 
-	if(d->tb_pgp->isOn()) {
+ 	// Request events
+ 	if (option.messageEvents) {
+ 		//m.addEvent(OfflineEvent);
+ 		//m.addEvent(DeliveredEvent);
+ 		//m.addEvent(DisplayedEvent);
+ 		m.addEvent(ComposingEvent);
+ 	}
+
+	if(d->act_pgp->isOn()) {
 		d->mle->setEnabled(false);
 		d->transid = d->pa->sendMessageEncrypted(m);
 		if(d->transid == -1) {
@@ -722,7 +860,16 @@ void ChatDlg::doSend()
 void ChatDlg::doneSend()
 {
 	appendMessage(d->m, true);
+	disconnect(d->mle, SIGNAL(textChanged()), d, SLOT(setComposing()));
 	d->mle->setText("");
+
+	// Reset composing timer
+	connect(d->mle, SIGNAL(textChanged()), d, SLOT(setComposing()));
+	if (d->composingTimer) {
+		delete d->composingTimer;
+		d->composingTimer = 0;
+		d->isComposing = false;
+	}
 }
 
 void ChatDlg::encryptedMessageSent(int x, bool b)
@@ -742,7 +889,22 @@ void ChatDlg::encryptedMessageSent(int x, bool b)
 
 void ChatDlg::incomingMessage(const Message &m)
 {
-	appendMessage(m);
+	if (m.body().isEmpty()) {
+		/* Event message */
+		if (m.containsEvent(CancelEvent))
+			updateContactIsComposing(false);
+		if (m.containsEvent(ComposingEvent))
+			updateContactIsComposing(true);
+	}
+	else {
+		// Normal message
+		// Check if user requests event messages
+		d->sendComposingEvents = m.containsEvent(ComposingEvent);
+		if (!m.eventId().isEmpty())
+			d->eventId = m.eventId();
+		updateContactIsComposing(false);
+		appendMessage(m);
+	}
 }
 
 void ChatDlg::appendSysMsg(const QString &str)
@@ -789,12 +951,12 @@ void ChatDlg::appendMessage(const Message &m, bool local)
 		if(encEnabled) {
 			appendSysMsg(QString("<icon name=\"psi/cryptoYes\"> ") + tr("Encryption Enabled"));
 			if(!local)
-				d->tb_pgp->setOn(true);
+				d->act_pgp->setOn(true);
 		}
 		else {
 			appendSysMsg(QString("<icon name=\"psi/cryptoNo\"> ") + tr("Encryption Disabled"));
 			if(!local) {
-				d->tb_pgp->setOn(false);
+				d->act_pgp->setOn(false);
 
 				// enable warning
 				d->warnSend = true;
@@ -805,7 +967,6 @@ void ChatDlg::appendMessage(const Message &m, bool local)
 
 	QString timestr;
 	QDateTime time = m.timeStamp();
-	//timestr.sprintf("%02d:%02d:%02d", time.time().hour(), time.time().minute(), time.time().second());
 	timestr = time.time().toString(LocalDate);
 
 	int y = d->log->contentsHeight() - d->log->visibleHeight();
@@ -883,6 +1044,42 @@ void ChatDlg::slotScroll()
 	d->log->scrollToBottom();
 }
 
+void ChatDlg::updateIsComposing(bool c)
+{
+	if (option.messageEvents && d->sendComposingEvents) {
+		// Don't send to offline resource
+		QPtrList<UserListItem> ul = d->pa->findRelavent(d->jid);
+		if(ul.isEmpty()) {
+			d->sendComposingEvents = false;
+			return;
+		}
+
+		QString rname = d->jid.resource();
+		UserListItem *u = ul.first();
+		if(rname.isEmpty() && !u->isAvailable() || u->userResourceList().find(rname) == u->userResourceList().end()) {
+			d->sendComposingEvents = false;
+			return;
+		}
+
+		// Send event message
+		Message m(d->jid);
+		m.setEventId(d->eventId);
+		if (c) {
+			m.addEvent(ComposingEvent);
+		}
+		else {
+			m.addEvent(CancelEvent);
+		}
+		d->pa->dj_sendMessage(m, false);
+	}
+}
+
+void ChatDlg::updateContactIsComposing(bool c)
+{
+	d->contactIsComposing = c;
+	updateCaption();
+}
+
 void ChatDlg::toggleSmallChat()
 {
 	if (d->smallChat)
@@ -894,23 +1091,25 @@ void ChatDlg::toggleSmallChat()
 
 void ChatDlg::toggleEncryption()
 {
-	d->tb_pgp->setOn(1-d->tb_pgp->isOn());
+	d->act_pgp->setOn( !d->act_pgp->isOn() );
 }
 
 void ChatDlg::buildMenu()
 {
+	// Dialog menu
 	d->pm_settings->clear();
-	d->pm_settings->insertItem("Toggle Compact/Full Size", this, SLOT(toggleSmallChat()));
-	d->pm_settings->insertItem(IconsetFactory::iconPtr("psi/clearChat")->pixmap(),"Clear Chat", this, SLOT(doClearButton()));
+	d->pm_settings->insertItem(tr("Toggle Compact/Full Size"), this, SLOT(toggleSmallChat()));
+
+	d->act_clear->addTo( d->pm_settings );
 	d->pm_settings->insertSeparator();
-	if (d->tb_pgp->isEnabled()) {
-		if (d->tb_pgp->isOn())
-			d->pm_settings->insertItem(IconsetFactory::icon("psi/cryptoYes").pixmap(),"Toggle Encryption", this, SLOT(toggleEncryption()));
-		else
-			d->pm_settings->insertItem(IconsetFactory::icon("psi/cryptoNo").pixmap(),"Toggle Encryption", this, SLOT(toggleEncryption()));
-	}
+
+	d->act_icon->addTo( d->pm_settings );
+	d->act_file->addTo( d->pm_settings );
+	d->act_pgp->addTo( d->pm_settings );
 	d->pm_settings->insertSeparator();
-	d->pm_settings->insertItem(IconsetFactory::iconPtr("psi/vCard")->pixmap(),"View User Profile", this, SLOT(doInfo()));
-	d->pm_settings->insertItem(IconsetFactory::iconPtr("psi/history")->pixmap(),"View History", this, SLOT(doHistory()));
+
+	d->act_info->addTo( d->pm_settings );
+	d->act_history->addTo( d->pm_settings );
 }
+
 #include "chatdlg.moc"
