@@ -28,6 +28,8 @@ using namespace XMPP;
 
 class PsiAccount;
 class UserListItem;
+class QDropEvent;
+class QDragEnterEvent;
 
 class ChatDlg : public QWidget
 {
@@ -55,11 +57,16 @@ protected:
 	void resizeEvent(QResizeEvent *);
 	void showEvent(QShowEvent *);
 	void windowActivationChange(bool);
+	void dropEvent(QDropEvent* event);
+	void dragEnterEvent(QDragEnterEvent* event);
 
 public slots:
 	void optionsUpdate();
 	void updateContact(const Jid &, bool);
 	void incomingMessage(const Message &);
+	// Avatars
+	//void updateAvatar();
+	//void updateAvatar(const Jid&);
 
 private slots:
 	void doInfo();
@@ -74,6 +81,8 @@ private slots:
 	void updatePGP();
 	void encryptedMessageSent(int, bool);
 	void slotScroll();
+	void updateIsComposing(bool);
+	void updateContactIsComposing(bool);
 	void toggleSmallChat();
 	void toggleEncryption();
 	void buildMenu();
