@@ -26,7 +26,8 @@
 #include"psievent.h"
 
 QString PROG_NAME = "Psi";
-QString PROG_VERSION = "0.9.2";
+//QString PROG_VERSION = "0.10-CVS" " (" __DATE__ ")"; //CVS Builds are dated
+QString PROG_VERSION = "0.9.3";
 
 #ifdef HAVE_CONFIG
 #include"config.h"
@@ -43,6 +44,8 @@ QString PROG_VERSION = "0.9.2";
 #include<qsound.h>
 #include<qobjectlist.h>
 #include<qlibrary.h>
+#include<qstylesheet.h>
+
 
 #include<stdio.h>
 
@@ -351,26 +354,7 @@ QString clipStatus(const QString &str, int width, int height)
 
 QString expandEntities(const QString &in)
 {
-	QString out;
-	int col = 0;
-
-	for(int i = 0; i < (int)in.length(); ++i) {
-		if(in[i] == '<')
-			out += "&lt;";
-		else if(in[i] == '>')
-			out += "&gt;";
-		else if(in[i] == '\"')
-			out += "&quot;";
-		else if(in[i] == '\'')
-			out += "&apos;";
-		else if(in[i] == '&')
-			out += "&amp;";
-		else
-			out += in[i];
-		++col;
-	}
-
-	return out;
+	return QStyleSheet::escape(in);
 }
 
 QString resolveEntities(const QString &in)
@@ -1247,13 +1231,21 @@ static void getSysInfo()
 	else if(v == Qt::WV_98)
 		os_str = "Windows 98";
 	else if(v == Qt::WV_Me)
-		os_str = "Windows ME";
+		os_str = "Windows Me";
+	else if(v == Qt::WV_DOS_based)
+		os_str = "Windows 9x/Me";
 	else if(v == Qt::WV_NT)
 		os_str = "Windows NT 4.x";
 	else if(v == Qt::WV_2000)
-		os_str = "Windows 2000 (NT5)";
+		os_str = "Windows 2000";
 	else if(v == Qt::WV_XP)
 		os_str = "Windows XP";
+#if QT_VERSION >= 0x030300
+	else if(v == Qt::WV_2003)
+		os_str = "Windows Server 2003";
+#endif
+	else if(v == Qt::WV_NT_based)
+		os_str = "Windows NT";
 #endif
 	sysinfo_done = TRUE;
 }
@@ -1411,12 +1403,24 @@ QString getHomeDir()
 
 QString getHistoryDir()
 {
-	return pathToProfile(activeProfile) + "/history";
+	QDir history(pathToProfile(activeProfile) + "/history");
+	if (!history.exists()) {
+		QDir profile(pathToProfile(activeProfile));
+		profile.mkdir("history");
+	}
+
+	return history.path();
 }
 
 QString getVCardDir()
 {
-	return pathToProfile(activeProfile) + "/vcard";
+	QDir vcard(pathToProfile(activeProfile) + "/vcard");
+	if (!vcard.exists()) {
+		QDir profile(pathToProfile(activeProfile));
+		profile.mkdir("vcard");
+	}
+
+	return vcard.path();
 }
 
 bool fileCopy(const QString &src, const QString &dest)
