@@ -30,6 +30,7 @@
 #include<qinputdialog.h>
 #include<qiconset.h>
 #include<qdragobject.h>
+#include<qfiledialog.h>
 #include"im.h"
 #include"common.h"
 #include"userlist.h"
@@ -37,6 +38,7 @@
 #include"psicon.h"
 #include"iconaction.h"
 #include"alerticon.h"
+//#include"avatars.h"
 
 //----------------------------------------------------------------------------
 // ContactProfile
@@ -244,6 +246,15 @@ ContactViewItem *ContactProfile::checkGroup(int type)
 	return 0;
 }
 
+// make a tooltip with account information
+QString ContactProfile::makeTip(bool trim, bool doLinkify) const
+{
+	if (d->cvi)
+		return "<qt> <center> <b>" + d->cvi->text(0) + " " + d->cvi->groupInfo() + "</b> </center>" + d->su.makeBareTip(trim,doLinkify) + "</qt>";
+	else
+		return d->su.makeTip(trim,doLinkify);
+}
+
 // check for user group
 ContactViewItem *ContactProfile::checkGroup(const QString &name)
 {
@@ -296,48 +307,72 @@ void ContactProfile::checkDestroyGroup(ContactViewItem *group)
 
 void ContactProfile::updateEntry(const UserListItem &u)
 {
-	Entry *e = findEntry(u.jid());
-	if(!e) {
-		e = new Entry;
-		d->roster.append(e);
-		e->u = u;
-	}
-	else {
-		e->u = u;
-		removeUnneededContactItems(e);
+	if (u.isSelf()) {
+		// Update the self item
+		d->su = u;
 
-		// update remaining items
-		QPtrListIterator<ContactViewItem> it(e->cvi);
-		for(ContactViewItem *i; (i = it.current()); ++it) {
-			i->setContact(&e->u);
-			if(!u.isAvailable())
-				i->stopAnimateNick();
+		// Show and/or update item if necessary
+		if (d->cv->isShowSelf() || d->su.userResourceList().count() > 1) {
+			if (d->self) {
+				updateSelf();
+			}
+			else {
+				addSelf();
+			}
+		}
+		else {
+			removeSelf();
 		}
 	}
+	else {
+		Entry *e = findEntry(u.jid());
+		if(!e) {
+			e = new Entry;
+			d->roster.append(e);
+			e->u = u;
+		}
+		else {
+			e->u = u;
+			removeUnneededContactItems(e);
 
-	deferredUpdateGroups();
-	addNeededContactItems(e);
+			// update remaining items
+			QPtrListIterator<ContactViewItem> it(e->cvi);
+			for(ContactViewItem *i; (i = it.current()); ++it) {
+				i->setContact(&e->u);
+				if(!u.isAvailable())
+					i->stopAnimateNick();
+			}
+		}
+
+		deferredUpdateGroups();
+		addNeededContactItems(e);
+	}
 }
 
-void ContactProfile::updateSelf(const UserListItem &u)
+void ContactProfile::updateSelf()
+{
+	if (d->self) {
+		d->self->setContact(&d->su);
+		if(!d->su.isAvailable())
+			d->self->stopAnimateNick();
+	}
+}
+
+void ContactProfile::addSelf()
 {
-	d->su = u;
 	if(!d->self) {
 		if(!d->cvi)
 			return;
 		d->self = new ContactViewItem(&d->su, this, d->cvi);
 	}
-	else {
-		d->self->setContact(&d->su);
-		if(!u.isAvailable())
-			d->self->stopAnimateNick();
-	}
 }
 
 void ContactProfile::removeSelf()
 {
-	delete d->self;
-	d->self = 0;
+	if (d->self) {
+		delete d->self;
+		d->self = 0;
+	}
 }
 
 ContactViewItem *ContactProfile::addContactItem(Entry *e, ContactViewItem *group_item)
@@ -797,18 +832,18 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 
 		QPopupMenu *am = new QPopupMenu(&pm);
 		am->insertItem(is->transportStatus("transport", STATUS_ONLINE), tr("Online Users"), 5);
-		am->insertItem(IconsetFactory::icon("psi/sendMessage"), tr("Send server message"), 1);
+		am->insertItem(IconsetFactory::iconPixmap("psi/sendMessage"), tr("Send server message"), 1);
 		am->insertSeparator();
-		am->insertItem(/*IconsetFactory::icon("psi/edit"),*/ tr("Set MOTD"), 2);
-		am->insertItem(/*IconsetFactory::icon("psi/edit/clear"),*/ tr("Update MOTD"), 3);
-		am->insertItem(IconsetFactory::icon("psi/remove"), tr("Delete MOTD"), 4);
+		am->insertItem(/*IconsetFactory::iconPixmap("psi/edit"),*/ tr("Set MOTD"), 2);
+		am->insertItem(/*IconsetFactory::iconPixmap("psi/edit/clear"),*/ tr("Update MOTD"), 3);
+		am->insertItem(IconsetFactory::iconPixmap("psi/remove"), tr("Delete MOTD"), 4);
 
 		QPopupMenu *bm = new QPopupMenu(&pm);
 		bm->insertItem(is->transportStatus("transport", STATUS_ONLINE), tr("Service &Discovery"), 9);
 		bm->insertSeparator();
-		bm->insertItem(IconsetFactory::icon("psi/sendMessage"), tr("New &blank message"), 6);
-		bm->insertItem(IconsetFactory::icon("psi/addContact"), tr("&Add a contact"), 7);
-		bm->insertItem(IconsetFactory::icon("psi/xml"), tr("&XML Console"), 10);
+		bm->insertItem(IconsetFactory::iconPixmap("psi/sendMessage"), tr("New &blank message"), 6);
+		bm->insertItem(IconsetFactory::iconPixmap("psi/addContact"), tr("&Add a contact"), 7);
+		bm->insertItem(IconsetFactory::iconPixmap("psi/xml"), tr("&XML Console"), 10);
 
 		const int status_start = 16;
 		QPopupMenu *sm = new QPopupMenu(&pm);
@@ -826,7 +861,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 
 		pm.insertSeparator();
 		pm.insertItem(tr("A&ctions"), bm);
-		pm.insertItem(IconsetFactory::icon("psi/account"), tr("&Modify Account..."), 0);
+		pm.insertItem(IconsetFactory::iconPixmap("psi/account"), tr("&Modify Account..."), 0);
 		pm.insertSeparator();
 		pm.insertItem(tr("&Admin"), am);
 
@@ -882,7 +917,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		QString gname = i->groupName();
 		QPopupMenu pm;
 
-		pm.insertItem(IconsetFactory::icon("psi/sendMessage"), tr("Send message to group"), 0);
+		pm.insertItem(IconsetFactory::iconPixmap("psi/sendMessage"), tr("Send message to group"), 0);
 		if(!option.lockdown.roster) {
 			// disable if it's not a user group
 			if(!online || i->groupType() != ContactViewItem::gUser || gname == ContactView::tr("Hidden")) {
@@ -895,8 +930,8 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 
 			d->cv->qa_ren->addTo(&pm);
 			pm.insertSeparator();
-			pm.insertItem(IconsetFactory::icon("psi/remove"), tr("Remove group"), 2);
-			pm.insertItem(IconsetFactory::icon("psi/remove"), tr("Remove group and contacts"), 3);
+			pm.insertItem(IconsetFactory::iconPixmap("psi/remove"), tr("Remove group"), 2);
+			pm.insertItem(IconsetFactory::iconPixmap("psi/remove"), tr("Remove group and contacts"), 3);
 		}
 
 		if(i->groupType() == ContactViewItem::gAgents) {
@@ -974,11 +1009,12 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		bool isPrivate = e ? e->u.isPrivate(): false;
 		bool isAgent = e ? e->u.isTransport() : false;
 		bool avail = e ? e->u.isAvailable() : false;
-
+		QString groupNameCache = ((ContactViewItem *)static_cast<QListViewItem *>(i)->parent())->groupName();
+		
 		QPopupMenu pm;
 
 		if(!self && !inList && !isPrivate && !option.lockdown.roster) {
-			pm.insertItem(IconsetFactory::icon("psi/addContact"), tr("Add/Authorize to contact list"), 10);
+			pm.insertItem(IconsetFactory::iconPixmap("psi/addContact"), tr("Add/Authorize to contact list"), 10);
 			if(!online)
 				pm.setItemEnabled(10, false);
 			pm.insertSeparator();
@@ -1036,7 +1072,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 		if(!isPrivate)
 			pm.insertItem(tr("Send message to"), s2m, 17);
 
-		d->cv->qa_chat->setIconSet(IconsetFactory::icon("psi/chat"));
+		d->cv->qa_chat->setIconSet(IconsetFactory::iconPixmap("psi/chat"));
 		d->cv->qa_chat->addTo(&pm);
 
 		if(!isPrivate)
@@ -1079,7 +1115,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 
 		if(!isAgent) {
 			pm.insertSeparator();
-			pm.insertItem(IconsetFactory::icon("psi/upload"), tr("Send &file"), 23);
+			pm.insertItem(IconsetFactory::iconPixmap("psi/upload"), tr("Send &file"), 23);
 			if(!online)
 				pm.setItemEnabled(23, false);
 		}
@@ -1096,7 +1132,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 				if(!online)
 					gm->setItemEnabled(id, false);
 			}
-			pm.insertItem(IconsetFactory::icon("psi/groupChat"), tr("Invite to"), gm, 14);
+			pm.insertItem(IconsetFactory::iconPixmap("psi/groupChat"), tr("Invite to"), gm, 14);
 			if(groupchats.isEmpty())
 				pm.setItemEnabled(14, false);
 		}
@@ -1127,7 +1163,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 					if(e->u.groups().isEmpty())
 						gm->setItemChecked(8, true);
 					else
-						g = ((ContactViewItem *)static_cast<QListViewItem *>(i)->parent())->groupName();
+						g = groupNameCache;
 
 					int n = 0;
 					gl.remove(ContactView::tr("Hidden"));
@@ -1149,7 +1185,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 					if(g == ContactView::tr("Hidden"))
 						gm->setItemChecked(n+base_group, true);
 					gm->insertSeparator();
-					gm->insertItem(IconsetFactory::icon("psi/edit/clear"), tr("&Create new..."), 9);
+					gm->insertItem(IconsetFactory::iconPixmap("psi/edit/clear"), tr("&Create new..."), 9);
 					pm.insertItem(tr("&Group"), gm, 5);
 
 					if(!online)
@@ -1176,9 +1212,9 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 
 			authm->insertItem(tr("Resend authorization to"), 6);
 			authm->insertItem(tr("Rerequest authorization from"), 11);
-			authm->insertItem(IconsetFactory::icon("psi/edit/delete"), tr("Remove authorization from"), 15);
+			authm->insertItem(IconsetFactory::iconPixmap("psi/edit/delete"), tr("Remove authorization from"), 15);
 
-			pm.insertItem (IconsetFactory::icon("psi/register"), tr("Authorization"), authm, 20);
+			pm.insertItem (IconsetFactory::iconPixmap("psi/register"), tr("Authorization"), authm, 20);
 			if(!online)
 				pm.setItemEnabled(20, false);
 		}
@@ -1195,16 +1231,24 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 			pm.insertSeparator();
 		}
 
+		// Avatars
+		//QPopupMenu *avpm = new QPopupMenu(&pm);
+		//d->cv->qa_assignAvatar->addTo(avpm);
+		//d->cv->qa_clearAvatar->setEnabled(d->pa->avatarFactory()->hasManualAvatar(u->jid()));
+		//d->cv->qa_clearAvatar->addTo(avpm);
+		//pm.insertItem(tr("&Avatar"), avpm);
+		//avpm->setEnabled(option.avatarsEnabled);
+
 		if(d->pa->psi()->pgp()) {
 			if(u->publicKeyID().isEmpty())
-				pm.insertItem(IconsetFactory::icon("psi/keySingle"), tr("Assign Open&PGP key"), 21);
+				pm.insertItem(IconsetFactory::iconPixmap("psi/keySingle"), tr("Assign Open&PGP key"), 21);
 			else
-				pm.insertItem(IconsetFactory::icon("psi/keySingle"), tr("Unassign Open&PGP key"), 22);
+				pm.insertItem(IconsetFactory::iconPixmap("psi/keySingle"), tr("Unassign Open&PGP key"), 22);
 		}
 
-		pm.insertItem(/*IconsetFactory::icon("psi/info"),*/ tr("Check &Status"), 4);
+		pm.insertItem(/*IconsetFactory::iconPixmap("psi/info"),*/ tr("Check &Status"), 4);
 
-		pm.insertItem(IconsetFactory::icon("psi/vCard"), tr("User &Info"), 12);
+		pm.insertItem(IconsetFactory::iconPixmap("psi/vCard"), tr("User &Info"), 12);
 
 		if(!isPrivate) {
 			d->cv->qa_hist->addTo(&pm);
@@ -1258,7 +1302,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 			if(online) {
 				// if we have groups, but we are setting to 'none', then remove that particular group
 				if(!u->groups().isEmpty()) {
-					QString gname = ((ContactViewItem *)static_cast<QListViewItem *>(i)->parent())->groupName();
+					QString gname = groupNameCache;
 					actionGroupRemove(u->jid(), gname);
 				}
 			}
@@ -1283,7 +1327,7 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 						}
 					}
 					if(!found) {
-						QString gname = ((ContactViewItem *)static_cast<QListViewItem *>(i)->parent())->groupName();
+						QString gname = groupNameCache;
 						actionGroupRemove(u->jid(), gname);
 						actionGroupAdd(u->jid(), newgroup);
 						break;
@@ -1409,7 +1453,8 @@ void ContactProfile::doContextMenu(ContactViewItem *i, const QPoint &pos)
 				if(n == n2) {
 					// remove the group of this cvi if there is one
 					if(!u->groups().isEmpty()) {
-						QString gname = ((ContactViewItem *)static_cast<QListViewItem *>(i)->parent())->groupName();
+						//QString gname = ((ContactViewItem *)static_cast<QListViewItem *>(i)->parent())->groupName();
+						QString gname = groupNameCache;
 						actionGroupRemove(u->jid(), gname);
 					}
 					// add the group
@@ -1612,6 +1657,17 @@ void ContactProfile::dragDrop(const QString &text, ContactViewItem *i)
 	}
 }
 
+void ContactProfile::dragDropFiles(const QStringList &files, ContactViewItem *i)
+{
+	if(files.isEmpty() || !d->pa->loggedIn() || i->type() != ContactViewItem::Contact)
+		return;
+
+	Entry *e = findEntry(i);
+	if(!e)
+		return;
+
+	actionSendFiles(e->u.jid(),files);
+}
 
 //----------------------------------------------------------------------------
 // ContactView
@@ -1728,6 +1784,7 @@ ContactView::ContactView(QWidget *parent, const char *name)
 	v_showAway = true;
 	v_showHidden = true;
 	v_showAgents = true;
+	v_showSelf = true;
 
 	// animation timer
 	d->animTimer = new QTimer(this);
@@ -1738,6 +1795,11 @@ ContactView::ContactView(QWidget *parent, const char *name)
 	connect(qa_send, SIGNAL(activated()), SLOT(doSendMessage()));
 	qa_ren = new IconAction("", /*"psi/edit/clear",*/ tr("Re&name"), QListView::Key_F2, this);
 	connect(qa_ren, SIGNAL(activated()), SLOT(doRename()));
+	// Avatars
+	//qa_assignAvatar = new IconAction("", tr("&Assign Avatar"),QKeySequence(), this);
+	//connect(qa_assignAvatar, SIGNAL(activated()), SLOT(doAssignAvatar()));
+	//qa_clearAvatar = new IconAction("", tr("&Clear Avatar"), QKeySequence(), this);
+	//connect(qa_clearAvatar, SIGNAL(activated()), SLOT(doClearAvatar()));
 	qa_chat = new IconAction("", "psi/chat", tr("Open &chat window"), QListView::CTRL+QListView::Key_C, this);
 	connect(qa_chat, SIGNAL(activated()), SLOT(doOpenChat()));
 	qa_hist = new IconAction("", "psi/history", tr("&History"), QListView::CTRL+QListView::Key_H, this);
@@ -1795,6 +1857,9 @@ void ContactView::maybeTip(const QPoint &pos)
 	QRect r(itemRect(i));
 	if(i->type() == ContactViewItem::Contact)
 		tip(r, i->u()->makeTip(true, false));
+	else if(i->type() == ContactViewItem::Profile) {
+		tip(r, i->contactProfile()->makeTip(true, false));
+	}
 	else
 		tip(r, i->text(0) + " " + i->groupInfo());
 }
@@ -1874,6 +1939,25 @@ void ContactView::setShowHidden(bool x)
 	}
 }
 
+void ContactView::setShowSelf(bool x)
+{
+	if (v_showSelf != x) {
+		v_showSelf = x;
+		showSelf(v_showSelf);
+
+		QPtrListIterator<ContactProfile> it(d->profiles);
+		for(ContactProfile *cp; (cp = it.current()); ++it) {
+			if (v_showSelf && ! cp->self()) {
+				cp->addSelf();
+			}
+			else if (!v_showSelf && cp->self() && cp->self()->u()->userResourceList().count() <= 1) {
+				cp->removeSelf();
+			}
+		}
+	}
+}
+
+
 void ContactView::setShowAgents(bool x)
 {
 	bool oldstate = v_showAgents;
@@ -2037,6 +2121,23 @@ void ContactView::doRename()
 	i->contactProfile()->scRename(i);
 }
 
+// Avatars
+//void ContactView::doAssignAvatar()
+//{
+//	// FIXME: Should check the supported filetypes dynamically
+//	QString file = QFileDialog::getOpenFileName("", tr("All files (*.png *.jpg *.gif)"), this, 0, tr("Choose an image"));
+//	if (!file.isNull()) {
+//		ContactViewItem *i = (ContactViewItem *)selectedItem();
+//		i->contactProfile()->psiAccount()->avatarFactory()->importManualAvatar(i->u()->jid(),file);
+//	}
+//}
+
+//void ContactView::doClearAvatar()
+//{
+//	ContactViewItem *i = (ContactViewItem *)selectedItem();
+//	i->contactProfile()->psiAccount()->avatarFactory()->removeManualAvatar(i->u()->jid());
+//}
+
 void ContactView::doEnter()
 {
 	ContactViewItem *i = (ContactViewItem *)selectedItem();
@@ -2107,7 +2208,7 @@ QDragObject *ContactView::dragObject()
 		return 0;
 
 	QDragObject *d = new QTextDrag(i->u()->jid().full(), this);
-	d->setPixmap(IconsetFactory::icon("status/online"), QPoint(8,8));
+	d->setPixmap(IconsetFactory::iconPixmap("status/online"), QPoint(8,8));
 	return d;
 }
 
@@ -2129,8 +2230,6 @@ public:
 	}
 
 	~Private() {
-		if ( icon )
-			icon->stop();
 	}
 
 	void initGroupState() {
@@ -2263,6 +2362,7 @@ ContactViewItem::ContactViewItem(UserListItem *u, ContactProfile *cp, ContactVie
 
 ContactViewItem::~ContactViewItem()
 {
+	setIcon( 0 );
 	delete d;
 }
 
@@ -2387,7 +2487,7 @@ void ContactViewItem::paintCell(QPainter *p, const QColorGroup & cg, int column,
 			UserResourceList::ConstIterator it = d->u->priority();
 			if(it != d->u->userResourceList().end()) {
 				if(d->u->isSecure((*it).name())) {
-					const QPixmap &pix = IconsetFactory::icon("psi/cryptoYes");
+					const QPixmap &pix = IconsetFactory::iconPixmap("psi/cryptoYes");
 					int y = (height() - pix.height()) / 2;
 					p->drawPixmap(x, y, pix);
 					x += 24;
@@ -2416,7 +2516,7 @@ void ContactViewItem::paintCell(QPainter *p, const QColorGroup & cg, int column,
 		int x = fm.width(text(column)) + (pix ? pix->width() : 0) + 8;
 
 		if(d->type == Profile) {
-			const QPixmap &pix = d->ssl ? IconsetFactory::icon("psi/cryptoYes") : IconsetFactory::icon("psi/cryptoNo");
+			const QPixmap &pix = d->ssl ? IconsetFactory::iconPixmap("psi/cryptoYes") : IconsetFactory::iconPixmap("psi/cryptoNo");
 			int y = (height() - pix.height()) / 2;
 			p->drawPixmap(x, y, pix);
 			x += 24;
@@ -2817,8 +2917,16 @@ bool ContactViewItem::acceptDrop(const QMimeSource *m) const
 			return false;
 	}
 
+	// Files. Note that the QTextDrag test has to come after QUriDrag.
+	if (d->type == Contact && QUriDrag::canDecode(m)) {
+		QStringList l;
+		if (QUriDrag::decodeLocalFiles(m,l) && !l.isEmpty())
+			return true;
+	}
+
 	if(!QTextDrag::canDecode(m))
 		return false;
+
 	QString str;
 	if(!QTextDrag::decode(m, str))
 		return false;
@@ -2841,9 +2949,21 @@ void ContactViewItem::dropped(QDropEvent *i)
 	if(!acceptDrop(i))
 		return;
 
-	QString text;
-	if(QTextDrag::decode(i, text))
-		d->cp->dragDrop(text, this);
+	// Files
+	if (QUriDrag::canDecode(i)) {
+		QStringList l;
+		if (QUriDrag::decodeLocalFiles(i,l) && !l.isEmpty()) {
+			d->cp->dragDropFiles(l, this);
+			return;
+		}
+	}
+
+	// Text
+	if(QTextDrag::canDecode(i)) {
+		QString text;
+		if(QTextDrag::decode(i, text))
+			d->cp->dragDrop(text, this);
+	}
 }
 
 int ContactViewItem::rtti() const
