@@ -64,7 +64,8 @@ public:
 
 	void updateEntry(const UserListItem &);
 	void removeEntry(const Jid &);
-	void updateSelf(const UserListItem &);
+	void updateSelf();
+	void addSelf();
 	void removeSelf();
 	void setAlert(const Jid &, const Icon *);
 	void clearAlert(const Jid &);
@@ -75,12 +76,15 @@ public:
 
 	void ensureVisible(const Jid &);
 	void clear();
+	
+	QString makeTip(bool trim, bool doLinkify) const;
 
 	ContactViewItem *checkGroup(int type);
 	ContactViewItem *checkGroup(const QString &name);
 
 	void setName(const char *);
 
+
 signals:
 	void actionDefault(const Jid &);
 	void actionRecvEvent(const Jid &);
@@ -104,6 +108,7 @@ signals:
 	void actionGroupRemove(const Jid &, const QString &);
 	void actionTest(const Jid &);
 	void actionSendFile(const Jid &);
+	void actionSendFiles(const Jid &, const QStringList &);
 	void actionDisco(const Jid &, const QString &);
 	void actionInvite(const Jid &, const QString &);
 	void actionAssignKey(const Jid &);
@@ -163,6 +168,7 @@ private:
 
 	friend class ContactViewItem;
 	void dragDrop(const QString &, ContactViewItem *);
+	void dragDropFiles(const QStringList &, ContactViewItem *);
 };
 
 // ContactView: the actual widget
@@ -177,12 +183,16 @@ public:
 	bool isShowAgents() const { return v_showAgents; }
 	bool isShowAway() const { return v_showAway; }
 	bool isShowHidden() const { return v_showHidden; }
+	bool isShowSelf() const { return v_showSelf; }
 
 	void clear();
 	void resetAnim();
 	QTimer *animTimer() const;
 
 	IconAction *qa_send, *qa_chat, *qa_ren, *qa_hist, *qa_logon, *qa_recv, *qa_rem;
+	// Avatars
+	//IconAction *qa_assignAvatar, *qa_clearAvatar;
+
 
 protected:
 	// reimplemented
@@ -195,12 +205,14 @@ signals:
 	void showAway(bool);
 	void showHidden(bool);
 	void showAgents(bool);
+	void showSelf(bool);
 
 public slots:
 	void setShowOffline(bool);
 	void setShowAgents(bool);
 	void setShowAway(bool);
 	void setShowHidden(bool);
+	void setShowSelf(bool);
 	void optionsUpdate();
 
 private slots:
@@ -220,6 +232,10 @@ private slots:
 	void doHistory();
 	void doLogon();
 	void doRemove();
+	
+	// Avatars
+	//void doAssignAvatar();
+	//void doClearAvatar();
 
 public:
 	class Private;
@@ -227,7 +243,7 @@ private:
 	Private *d;
 
 	QPoint mousePressPos; // store pressed position, idea taken from Licq
-	bool v_showOffline, v_showAgents, v_showAway, v_showHidden;
+	bool v_showOffline, v_showAgents, v_showAway, v_showHidden, v_showSelf;
 	bool lcto_active; // double click active?
 	QPoint lcto_pos;
 	QListViewItem *lcto_item;
