@@ -48,6 +48,28 @@ static int calcShift(Q_LLONG big)
 	return 0;
 }
 
+static int calcComplement(Q_LLONG big, int shift)
+{
+	int block = 1 << shift;
+	Q_LLONG rem = big % block;
+	if(rem == 0)
+		return 0;
+	else
+		return (block - (int)rem);
+}
+
+static int calcTotalSteps(Q_LLONG big, int shift)
+{
+	if(big < 1)
+		return 0;
+	return ((big - 1) >> shift) + 1;
+}
+
+static int calcProgressStep(Q_LLONG big, int complement, int shift)
+{
+	return ((big + complement) >> shift);
+}
+
 static QStringList *activeFiles = 0;
 
 static void active_file_add(const QString &s)
@@ -113,6 +135,7 @@ public:
 	bool sending;
 	QFile f;
 	int shift;
+	int complement;
 	QString activeFile;
 };
 
@@ -129,6 +152,7 @@ FileTransferHandler::FileTransferHandler(PsiAccount *pa, FileTransfer *ft)
 		d->fileSize = ft->fileSize();
 		d->desc = ft->description();
 		d->shift = calcShift(d->fileSize);
+		d->complement = calcComplement(d->fileSize, d->shift);
 		d->ft = ft;
 		Jid proxy = d->pa->userAccount().dtProxy;
 		if(proxy.isValid())
@@ -164,6 +188,7 @@ void FileTransferHandler::send(const XMPP::Jid &to, const QString &fname, const
 	d->fileSize = fi.size(); // TODO: large file support
 	d->desc = desc;
 	d->shift = calcShift(d->fileSize);
+	d->complement = calcComplement(d->fileSize, d->shift);
 
 	d->ft = d->pa->client()->fileTransferManager()->createTransfer();
 	Jid proxy = d->pa->userAccount().dtProxy;
@@ -212,7 +237,7 @@ Q_LLONG FileTransferHandler::offset() const
 
 int FileTransferHandler::totalSteps() const
 {
-	return (d->fileSize >> d->shift);
+	return calcTotalSteps(d->fileSize, d->shift);
 }
 
 bool FileTransferHandler::resumeSupported() const
@@ -386,7 +411,7 @@ void FileTransferHandler::ft_bytesWritten(int x)
 		}
 		else
 			QTimer::singleShot(0, this, SLOT(trySend()));
-		progress(d->sent >> d->shift, d->sent);
+		progress(calcProgressStep(d->sent, d->complement, d->shift), d->sent);
 	}
 }
 
@@ -433,7 +458,7 @@ void FileTransferHandler::doFinish()
 		delete d->ft;
 		d->ft = 0;
 	}
-	progress(d->sent >> d->shift, d->sent);
+	progress(calcProgressStep(d->sent, d->complement, d->shift), d->sent);
 }
 
 void FileTransferHandler::mapSignals()
@@ -466,7 +491,16 @@ public:
 	QTimer t;
 };
 
-FileRequestDlg::FileRequestDlg(const Jid &jid, PsiCon *psi, PsiAccount *pa)
+
+FileRequestDlg::FileRequestDlg(const Jid &j, PsiCon *psi, PsiAccount *pa) 
+:FileTransUI(0, 0, false, psi_dialog_flags | WDestructiveClose)
+{
+	QStringList l;
+	FileRequestDlg(j, psi, pa, l);
+}
+
+
+FileRequestDlg::FileRequestDlg(const Jid &jid, PsiCon *psi, PsiAccount *pa, const QStringList& files)
 :FileTransUI(0, 0, false, psi_dialog_flags | WDestructiveClose)
 {
 	d = new Private;
@@ -493,7 +527,9 @@ FileRequestDlg::FileRequestDlg(const Jid &jid, PsiCon *psi, PsiAccount *pa)
 	setTabOrder(d->te, pb_stop);
 
 	setCaption(tr("Send File"));
+#ifndef Q_WS_MAC
 	setIcon(IconsetFactory::icon("psi/upload"));
+#endif
 
 	le_to->setText(d->jid.full());
 	le_to->setReadOnly(false);
@@ -508,7 +544,30 @@ FileRequestDlg::FileRequestDlg(const Jid &jid, PsiCon *psi, PsiAccount *pa)
 
 	d->te->setFocus();
 	d->psi->dialogRegister(this);
-	QTimer::singleShot(0, this, SLOT(chooseFile()));
+
+	if (files.isEmpty()) {
+		QTimer::singleShot(0, this, SLOT(chooseFile()));
+	}
+	else {
+		// TODO: Once sending of multiple files is supported, change this
+		QFileInfo fi(files.first());
+
+		// Check if the file is legal
+		if(!fi.exists()) {
+			QMessageBox::critical(this, tr("Error"), QString("The file '%1' does not exist.").arg(files.first()));
+			QTimer::singleShot(0, this, SLOT(reject()));
+			return;
+		}
+		if(fi.isDir()) {
+			QMessageBox::critical(this, tr("Error"), tr("Sending folders is not supported."));
+			QTimer::singleShot(0, this, SLOT(reject()));
+			return;
+		}
+		
+		lastPath = fi.dirPath();
+		le_fname->setText(QDir::convertSeparators(fi.filePath()));
+		lb_size->setText(tr("%1 byte(s)").arg(fi.size())); // TODO: large file support
+	}
 }
 
 FileRequestDlg::FileRequestDlg(const QDateTime &ts, FileTransfer *ft, PsiAccount *pa)
@@ -544,7 +603,9 @@ FileRequestDlg::FileRequestDlg(const QDateTime &ts, FileTransfer *ft, PsiAccount
 
 	lb_to->setText(tr("From:"));
 	setCaption(tr("Receive File"));
+#ifndef Q_WS_MAC
 	setIcon(IconsetFactory::icon("psi/download"));
+#endif
 
 	le_to->setText(d->jid.full());
 	le_fname->setText(d->fileName);
@@ -1445,7 +1506,9 @@ FileTransDlg::FileTransDlg(PsiCon *psi)
 	connect(&d->t, SIGNAL(timeout()), SLOT(updateItems()));
 
 	setCaption(tr("Transfer Manager"));
+#ifndef Q_WS_MAC
 	setIcon(IconsetFactory::icon("psi/filemanager"));
+#endif
 
 	QVBoxLayout *vb = new QVBoxLayout(this, 11, 6);
 	d->lv = new FileTransView(this);
