@@ -72,7 +72,9 @@ InfoDlg::InfoDlg(int type, const Jid &j, const VCard &vcard, PsiAccount *pa, QWi
 	te_desc->setTextFormat(QTextEdit::PlainText);
 
 	setCaption(d->jid.full());
+#ifndef Q_WS_MAC
 	setIcon(IconsetFactory::icon("psi/vCard"));
+#endif
 
 	connect(pb_refresh, SIGNAL(clicked()), this, SLOT(doRefresh()));
 	connect(te_desc, SIGNAL(textChanged()), this, SLOT(textChanged()));
