@@ -31,6 +31,7 @@
 #include<qsocketdevice.h>
 #include<qcombobox.h>
 #include<qcheckbox.h>
+#include<qmessagebox.h>
 #include<qca.h>
 #include<stdlib.h>
 #include<time.h>
@@ -55,6 +56,7 @@
 using namespace XMPP;
 
 QTranslator *trans;
+QTranslator *qttrans;
 QString curLang = "en";
 QString curLangName = QT_TR_NOOP("language_name");
 
@@ -62,6 +64,7 @@ void setLang(const QString &lang)
 {
 	//printf("changing lang: [%s]\n", lang.latin1());
 	trans->clear();
+	qttrans->clear();
 	if(lang == "en") {
 		curLang = lang;
 		curLangName = "English";
@@ -77,6 +80,8 @@ void setLang(const QString &lang)
 		if(!QFile::exists(*it))
 			continue;
 		if(trans->load("psi_" + lang, *it)) {
+			// try to load qt library translation
+			qttrans->load("qt_" + lang, *it);
 			curLang = lang;
 			return;
 		}
@@ -173,12 +178,28 @@ PsiMain::PsiMain(QObject *par)
 	setLang(lastLang);
 
 	if(autoOpen && !lastProfile.isEmpty() && profileExists(lastProfile)) {
+		// Auto-open the last profile
 		activeProfile = lastProfile;
 		QTimer::singleShot(0, this, SLOT(sessionStart()));
 	}
 	else {
-		autoOpen = false;
-		QTimer::singleShot(0, this, SLOT(chooseProfile()));
+		if (!lastProfile.isEmpty() && !getProfilesList().isEmpty()) {
+			// Select a profile
+			QTimer::singleShot(0, this, SLOT(chooseProfile()));
+		}
+		else {
+			// Create & open the default profile
+			if (!profileExists("default") && !profileNew("default")) {
+				QMessageBox::critical(0, tr("Error"), 
+					tr("There was an error creating the default profile."));
+				QTimer::singleShot(0, this, SLOT(bail()));
+			}
+			else {
+				lastProfile = activeProfile = "default";
+				autoOpen = true;
+				QTimer::singleShot(0, this, SLOT(sessionStart()));
+			}
+		}
 	}
 }
 
@@ -373,6 +394,9 @@ int main(int argc, char *argv[])
 	trans = new QTranslator(0);
 	app->installTranslator(trans);
 
+	qttrans = new QTranslator(0);
+	app->installTranslator(qttrans);
+
 	// need SHA1 for Iconset sound
 	if(!QCA::isSupported(QCA::CAP_SHA1))
 		QCA::insertProvider(XMPP::createProviderHash());
@@ -385,6 +409,9 @@ int main(int argc, char *argv[])
 	app->removeTranslator(trans);
 	delete trans;
 	trans = 0;
+	app->removeTranslator(qttrans);
+	delete qttrans;
+	qttrans = 0;
 
 	delete app;
 
