@@ -39,13 +39,6 @@ class QLabel;
 class QVBoxLayout;
 class QMenuBar;
 
-class MPushButton;
-class PopupAction;
-class MLabel;
-class MTray;
-class MAction;
-class EventNotifierAction;
-
 class MainWin : public QMainWindow
 {
 	Q_OBJECT
@@ -56,59 +49,18 @@ public:
 	void setWindowOpts(bool onTop, bool asTool);
 	void setUseDock(bool);
 	void setInfo(const QString &);
-	void setUsingSSL(bool);
-
-	ContactView *cvlist;
 
+	// evil stuff! remove ASAP!!
 	QStringList actionList;
 	QMap<QString, QAction*> actions;
+
+	ContactView *cvlist;
 	QPtrList<PsiToolBar> toolbars;
 
 	void buildToolbars();
 	void saveToolbarsPositions();
 	PsiCon *psiCon() const;
 
-private:
-	bool onTop, asTool;
-	QPopupMenu *statusMenu, *optionsMenu;
-	int sbState;
-	QString nickname;
-	MTray *tray;
-	QPopupMenu *trayMenu;
-	QString statusTip;
-
-	MAction *lw_act, *add_act, *man_act, *actDisco;
-
-	PopupAction *optionsButton, *statusButton;
-
-	IconAction *showOffline, *showAway, *showAgents, *showHidden;
-
-	IconAction *actReadme, *actAbout, *actAboutQt, *actOnlineHelp, *actBugReport, *actNewMessage,
-		   *actJoinGroupchat, *actAccountSetup, *actOptions, *actToolbars,
-		   *actChangeProfile, *actPlaySounds, *actQuit, *actTip, *actFileTrans;
-
-	IconAction *statusOffline, *statusOnline, *statusAway, *statusXa,
-		   *statusDnd, *statusInvisible, *statusChat;
-	QActionGroup *statusGroup;
-
-	QVBoxLayout *vb_main;
-
-	EventNotifierAction *eventNotifier;
-	IconLabel *lb_ssl;
-	bool usingSSL;
-
-	PsiCon *psi;
-
-	Icon *nextAnim;
-	int nextAmount;
-
-	bool askQuit();
-
-	void updateCaption();
-	void updateTray();
-
-	QString infoString;
-
 protected:
 	void closeEvent(QCloseEvent *);
 	void keyPressEvent(QKeyEvent *);
@@ -132,11 +84,12 @@ private slots:
 	void buildStatusMenu();
 	void buildOptionsMenu();
 	void buildTrayMenu();
+	void buildMainMenu();
+	void buildToolsMenu();
 
 	void setTrayToolTip(int);
 
 	void activatedStatusAction(int);
-	void activatedOptionsMenu(int);
 
 	void trayClicked(const QPoint &, int);
 	void trayDoubleClicked();
@@ -157,13 +110,16 @@ private slots:
 	void actBugReportActivated ();
 	void actAboutActivated ();
 	void actAboutQtActivated ();
-	void actPlaySoundsActivated ();
+	void actPlaySoundsActivated (bool);
 	void actTipActivated();
 
 	bool showDockMenu(const QPoint &);
 	void dockActivated();
 
+	void registerAction( IconAction * );
+
 public slots:
+	void setIcon(const QPixmap&);
 	void showNoFocus();
 
 	void decorateButton(int);
@@ -174,11 +130,18 @@ public slots:
 	void setTrayToolTip(const Status &);
 
 private:
-	int lastStatus;
-#ifdef Q_WS_MAC
-	QMenuBar *gm;
-#endif
+	void buildGeneralMenu(QPopupMenu *);
 	QString numEventsString(int) const;
+
+	bool askQuit();
+
+	void updateCaption();
+	void updateTray();
+
+private:
+	class Private;
+	Private *d;
+	friend class Private;
 };
 
 #endif
