@@ -21,11 +21,13 @@
 #include "common.h"
 #include "mainwin_p.h"
 
+#include <qapplication.h>
 #include <qstyle.h>
 #include <qtoolbar.h>
 #include <qtimer.h>
 #include <qsignalmapper.h>
 #include <qobjectlist.h>
+#include <qpixmapcache.h>
 #include "psiaccount.h"
 #include "iconwidget.h"
 #include "alerticon.h"
@@ -77,12 +79,14 @@ void PopupActionButton::setIcon(Icon *i, bool st)
 	}
 
 	icon = i;
-
 	showText = st;
-	pixmapUpdated(i->pixmap());
 
-	connect(icon, SIGNAL(pixmapChanged(const QPixmap &)), SLOT(pixmapUpdated(const QPixmap &)));
-	icon->activated();
+	if ( icon ) {
+		pixmapUpdated(icon->pixmap());
+
+		connect(icon, SIGNAL(pixmapChanged(const QPixmap &)), SLOT(pixmapUpdated(const QPixmap &)));
+		icon->activated();
+	}
 }
 
 void PopupActionButton::pixmapUpdated(const QPixmap &pix)
@@ -166,16 +170,14 @@ void PopupActionButton::drawButtonLabel(QPainter *p)
 class PopupAction::Private : public QObject
 {
 public:
-	QPopupMenu *menu;
 	QSizePolicy size;
 	QPtrList<PopupActionButton> buttons;
 	Icon *icon;
 	bool showText;
 
-	Private (QPopupMenu *_menu, QObject *parent)
+	Private (QObject *parent)
 	: QObject (parent)
 	{
-		menu = _menu;
 		icon = 0;
 		showText = true;
 	}
@@ -189,9 +191,10 @@ public:
 };
 
 PopupAction::PopupAction (const QString &label, QPopupMenu *_menu, QObject *parent, const char *name)
-: QAction (label, label, 0, parent, name)
+: IconAction (label, label, 0, parent, name)
 {
-	d = new Private (_menu, this);
+	d = new Private (this);
+	setPopup( _menu );
 }
 
 void PopupAction::setSizePolicy (const QSizePolicy &p)
@@ -220,11 +223,11 @@ void PopupAction::setIcon (const Icon *icon, bool showText, bool alert)
 		else
 			d->icon = new AlertIcon(icon);
 
-		QAction::setIconSet(*icon);
+		IconAction::setIconSet(*icon);
 	}
 	else {
 		d->icon = 0;
-		QAction::setIconSet(QIconSet());
+		IconAction::setIconSet(QIconSet());
 	}
 
 	for ( QPtrListIterator<PopupActionButton> it(d->buttons); it.current(); ++it ) {
@@ -239,7 +242,7 @@ void PopupAction::setIcon (const Icon *icon, bool showText, bool alert)
 
 void PopupAction::setText (const QString &text)
 {
-	QAction::setText (text);
+	IconAction::setText (text);
 	for ( QPtrListIterator<PopupActionButton> it(d->buttons); it.current(); ++it ) {
 		PopupActionButton *btn = it.current();
 		btn->setText (text);
@@ -250,13 +253,17 @@ bool PopupAction::addTo (QWidget *w)
 {
 	if ( w->inherits("QPopupMenu") ) {
 		QPopupMenu *m = (QPopupMenu*)w;
-		m->insertItem (iconSet(), menuText(), d->menu);
+#ifndef Q_WS_MAC
+		m->insertItem (iconSet(), menuText(), popup());
+#else
+		m->insertItem (QIconSet(), menuText(), popup());
+#endif
 	}
 	else if ( w->inherits("QToolBar") ) {
 		QCString bname = name() + QCString("_action_button");
 		PopupActionButton *btn = new PopupActionButton ( (QToolBar*)w, bname );
 		d->buttons.append ( btn );
-		btn->setPopup ( d->menu );
+		btn->setPopup ( popup() );
 		btn->setText ( text() );
 		btn->setIcon ( d->icon, d->showText );
 		btn->setSizePolicy ( d->size );
@@ -278,13 +285,33 @@ void PopupAction::objectDestroyed ()
 
 void PopupAction::setEnabled (bool e)
 {
-	QAction::setEnabled (e);
+	IconAction::setEnabled (e);
 	for ( QPtrListIterator<PopupActionButton> it(d->buttons); it.current(); ++it ) {
 		PopupActionButton *btn = it.current();
 		btn->setEnabled (e);
 	}
 }
 
+IconAction *PopupAction::copy() const
+{
+	PopupAction *act = new PopupAction(text(), popup(), 0, name());
+
+	*act = *this;
+
+	return act;
+}
+
+PopupAction &PopupAction::operator=( const PopupAction &from )
+{
+	*( (IconAction *)this ) = from;
+
+	d->size = from.d->size;
+	setIcon( from.d->icon );
+	d->showText = from.d->showText;
+
+	return *this;
+}
+
 //----------------------------------------------------------------------------
 // MLabel -- a clickable label
 //----------------------------------------------------------------------------
@@ -333,19 +360,22 @@ public:
 	TrayIcon *ti;
 	Icon *icon;
 
-	QPixmap makeIcon(const QImage &_in);
+	QPixmap makeIcon();
+	QRgb pixelBlend(QRgb p1, QRgb p2);
 };
 
-MTray::MTray(const QImage &icon, const QString &tip, QPopupMenu *popup, QObject *parent)
+MTray::MTray(const QString &tip, QPopupMenu *popup, QObject *parent)
 :QObject(parent)
 {
 	d = new Private;
 
-	d->ti = new TrayIcon(d->makeIcon(icon), tip, popup);
+	d->ti = new TrayIcon(d->makeIcon(), tip, popup);
 	d->ti->setWMDock(option.isWMDock);
 	connect(d->ti, SIGNAL(clicked(const QPoint &, int)), SIGNAL(clicked(const QPoint &, int)));
 	connect(d->ti, SIGNAL(doubleClicked(const QPoint &)), SIGNAL(doubleClicked(const QPoint &)));
 	connect(d->ti, SIGNAL(closed()), SIGNAL(closed()));
+	connect(qApp, SIGNAL(newTrayOwner()), d->ti, SLOT(newTrayOwner()));
+	connect(qApp, SIGNAL(trayOwnerDied()), d->ti, SLOT(hide()));
 	d->ti->show();
 }
 
@@ -409,125 +439,99 @@ void MTray::hide()
 	d->ti->hide();
 }
 
-QPixmap MTray::Private::makeIcon(const QImage &_in)
+
+// a function to blend 2 pixels taking their alpha channels
+// into consideration
+// p1 is in the 1st layer, p2 is in the 2nd layer (over p1)
+QRgb MTray::Private::pixelBlend(QRgb p1, QRgb p2)
 {
+	int a2 = qAlpha(p2);
+	if (a2 == 255) return p2; // don't calculate anything if p2 is completely opaque
+	int a1 = qAlpha(p1);
+	double prop1 = double(a1*(255-a2))/double(255*255);
+	double prop2 = double(a2)/255.0;
+	int r = int( qRed(p1)*prop1 + qRed(p2)*prop2 );
+	int g = int( qGreen(p1)*prop1 + qGreen(p2)*prop2 );
+	int b = int( qBlue(p1)*prop1 + qBlue(p2)*prop2 );
+	return qRgba(r, g, b, (a1>a2) ? a1:a2);
+}
+
+
+QPixmap MTray::Private::makeIcon()
+{
+	if ( !icon )
+		return QPixmap();
+
 #ifdef Q_WS_X11
-	/*if(1) {
-		QImage real = wmdock[0];
-		real.detach();
-		QImage in = _in;
-		in.detach();
+	// on X11, the KDE dock is 22x22.  let's make our icon "seem" bigger.
+	QImage real(22,22,32);
+	QImage in = icon->image();
+	in.detach();
+	real.setAlphaBuffer(true);
 
-		// make sure it is no bigger than 16x16
-		if(in.width() > 16 || in.height() > 16)
-			in = in.smoothScale(16,16);
+	// make sure it is no bigger than 16x16
+	if(in.width() > 16 || in.height() > 16)
+		in = in.smoothScale(16,16);
 
-		int xo = 40;
-		int yo = 40;
+	int xo = (real.width() - in.width()) / 2;
+	int yo = (real.height() - in.height()) / 2;
 
-		int n, n2;
+	int n, n2;
 
-		// draw a border
-		for(n = 0; n < 22; ++n) {
-			real.setPixel(n + xo, yo, qRgb(0x80,0x80,0x80));
-			real.setPixel(n + xo, yo + 21, qRgb(0x00,0x00,0x00));
-			real.setPixel(xo, yo + n, qRgb(0x80,0x80,0x80));
-			real.setPixel(xo + 21, yo + n, qRgb(0x00,0x00,0x00));
-		}
+	// clear the output and make it transparent
+	// deprecates real.fill(0)
+	for(n2 = 0; n2 < real.height(); ++n2)
+		for(n = 0; n < real.width(); ++n)
+			real.setPixel(n, n2, qRgba(0,0,0,0));
 
-		xo += 3;
-		yo += 3;
-
-		// draw a dropshadow
-		for(n2 = 0; n2 < in.height(); ++n2) {
-			for(n = 0; n < in.width(); ++n) {
-				if(qAlpha(_in.pixel(n,n2))) {
-					real.setPixel(n + xo + 2, n2 + yo + 2, qRgb(0x80,0x80,0x80));
-				}
+	// draw a dropshadow
+	for(n2 = 0; n2 < in.height(); ++n2) {
+		for(n = 0; n < in.width(); ++n) {
+			if(int a = qAlpha(in.pixel(n,n2))) {
+				int x = n + xo + 2;
+				int y = n2 + yo + 2;
+				real.setPixel(x, y, qRgba(0x80,0x80,0x80,a));
 			}
 		}
-		for(n2 = 0; n2 < in.height(); ++n2) {
-			for(n = 0; n < in.width(); ++n) {
-				if(qAlpha(_in.pixel(n,n2))) {
-					real.setPixel(n + xo, n2 + yo, _in.pixel(n,n2));
-				}
-			}
-		}
-
-		QPixmap icon;
-		icon.convertFromImage(real);
-		return icon;
 	}
-	else {*/
-		// on X11, the KDE dock is 22x22.  let's make our icon "seem" bigger.
-		QImage real(22,22,32);
-		//QImage in = _in.convertToImage();
-		QImage in = _in;
-		in.detach();
-		real.fill(0);
-		real.setAlphaBuffer(true);
 
-		// make sure it is no bigger than 16x16
-		if(in.width() > 16 || in.height() > 16)
-			in = in.smoothScale(16,16);
-
-		int xo = (real.width() - in.width()) / 2;
-		int yo = (real.height() - in.height()) / 2;
-
-		int n, n2;
-
-		// draw a border
-		/*for(n = 0; n < 22; ++n) {
-			real.setPixel(n,  0, qRgb(0x80,0x80,0x80));
-			real.setPixel(n, 21, qRgb(0x00,0x00,0x00));
-			real.setPixel( 0, n, qRgb(0x80,0x80,0x80));
-			real.setPixel(21, n, qRgb(0x00,0x00,0x00));
-		}*/
-
-		// draw a dropshadow
-		for(n2 = 0; n2 < in.height(); ++n2) {
-			for(n = 0; n < in.width(); ++n) {
-				if(qAlpha(in.pixel(n,n2))) {
-					int x = n + xo + 2;
-					int y = n2 + yo + 2;
-					real.setPixel(x, y, qRgba(0x80,0x80,0x80,0xff));
-				}
-			}
-		}
-		// draw the image
-		for(n2 = 0; n2 < in.height(); ++n2) {
-			for(n = 0; n < in.width(); ++n) {
-				if(qAlpha(in.pixel(n,n2))) {
-					QRgb c = in.pixel(n, n2);
-					real.setPixel(n + xo, n2 + yo, qRgba(qRed(c), qGreen(c), qBlue(c), 0xff));
-				}
-			}
-		}
-		// create the alpha layer
-		for(n2 = real.height()-2; n2 >= 0; --n2) {
-			for(n = 0; n < real.width(); ++n) {
-				uint c = real.pixel(n, n2);
-				if(c > 0) {
-					QRgb old = real.pixel(n, n2);
-					real.setPixel(n, n2, qRgba(qRed(old), qGreen(old), qBlue(old), 0xff));
-				}
+	// draw the image
+	for(n2 = 0; n2 < in.height(); ++n2) {
+		for(n = 0; n < in.width(); ++n) {
+			if(qAlpha(in.pixel(n,n2))) {
+				QRgb pold = real.pixel(n + xo , n2 + yo);
+				QRgb pnew = in.pixel(n , n2);
+				real.setPixel(n + xo, n2 + yo, pixelBlend(pold, pnew));
 			}
 		}
+	}
 
-		QPixmap icon;
-		icon.convertFromImage(real);
-		return icon;
-	//}
+	QPixmap pixmap;
+	pixmap.convertFromImage(real);
+	return pixmap;
 #else
-	QPixmap icon;
-	icon.convertFromImage(_in);
-	return icon;
+	return icon->pixmap();
 #endif
 }
 
 void MTray::animate()
 {
-	d->ti->setIcon(d->makeIcon(d->icon->image()));
+#ifdef Q_WS_X11
+	if ( !d->icon )
+		return;
+
+	QString cachedName = "PsiTray/" + option.defaultRosterIconset + "/" + d->icon->name() + "/" + QString::number( d->icon->framenumber() );
+
+	QPixmap p;
+	if ( !QPixmapCache::find(cachedName, p) ) {
+		p = d->makeIcon();
+		QPixmapCache::insert( cachedName, p );
+	}
+
+	d->ti->setIcon(p);
+#else
+	d->ti->setIcon( d->makeIcon() );
+#endif
 }
 
 //----------------------------------------------------------------------------
@@ -589,13 +593,17 @@ bool MAction::addTo(QWidget *w)
 	if ( w->inherits("QPopupMenu") )
 	{
 		QPopupMenu *menu = (QPopupMenu*)w;
+        QIconSet iconset;
+#ifndef Q_WS_MAC
+		iconset = iconSet();
+#endif
 		if ( d->psi->accountList(TRUE).count() < 2 ) {
-			menu->insertItem ( iconSet(), menuText(), this, SLOT(itemActivated(int)), 0, d->id*1000 + 0 );
+			menu->insertItem ( iconset, menuText(), this, SLOT(itemActivated(int)), 0, d->id*1000 + 0 );
 			menu->setItemEnabled (d->id*1000 + 0, isEnabled());
 			menu->setItemParameter ( d->id*1000 + 0, 0 );
 		}
 		else
-			menu->insertItem(iconSet(), menuText(), d->subMenu(w));
+			menu->insertItem(iconset, menuText(), d->subMenu(w));
 	}
 	else
 		return IconAction::addTo(w);
@@ -612,17 +620,6 @@ void MAction::addingToolButton(IconToolButton *btn)
 		btn->setPopup ( d->subMenu(btn) );
 	else
 		btn->setPopup (0);
-
-	btn->setPopupDelay (1); // the popup will be displayed immediately
-
-	QString tt, str = menuText();
-	for (int i = 0; i < (int)str.length(); i++)
-		if ( str[i] == '&' && str[i+1] != '&' )
-			continue;
-		else
-			tt += str[i];
-
-	QToolTip::add (btn, tt);
 }
 
 void MAction::itemActivated(int n)
@@ -652,6 +649,22 @@ void MAction::numAccountsChanged()
 	}
 }
 
+IconAction *MAction::copy() const
+{
+	MAction *act = new MAction(text(), d->id, d->psi, 0);
+
+	*act = *this;
+
+	return act;
+}
+
+MAction &MAction::operator=( const MAction &from )
+{
+	*( (IconAction *)this ) = from;
+
+	return *this;
+}
+
 //----------------------------------------------------------------------------
 // SpacerAction
 //----------------------------------------------------------------------------
@@ -673,11 +686,12 @@ public:
 };
 
 SpacerAction::SpacerAction(QObject *parent, const char *name)
-: QAction(parent, name)
+: IconAction(parent, name)
 {
 	d = new Private;
 	setText(tr("<Spacer>"));
 	setMenuText(tr("<Spacer>"));
+	setWhatsThis (tr("Spacer provides spacing to separate actions"));
 }
 
 SpacerAction::~SpacerAction()
@@ -695,6 +709,47 @@ bool SpacerAction::addTo(QWidget *w)
 	return false;
 }
 
+IconAction *SpacerAction::copy() const
+{
+	return new SpacerAction( 0 );
+}
+
+//----------------------------------------------------------------------------
+// SeparatorAction
+//----------------------------------------------------------------------------
+
+class SeparatorAction::Private
+{
+public:
+	Private() { }
+
+	QAction *separator;
+};
+
+SeparatorAction::SeparatorAction( QObject *parent, const char *name )
+	: IconAction( tr("<Separator>"), tr("<Separator>"), 0, parent, name )
+{
+	d = new Private();
+	d->separator = new QAction(this, "qt_separator_action");
+
+	setWhatsThis (tr("Separator"));
+}
+
+SeparatorAction::~SeparatorAction()
+{
+	delete d;
+}
+
+bool SeparatorAction::addTo (QWidget *w)
+{
+	return d->separator->addTo( w );
+}
+
+IconAction *SeparatorAction::copy() const
+{
+	return new SeparatorAction( 0 );
+}
+
 //----------------------------------------------------------------------------
 // EventNotifierAction
 //----------------------------------------------------------------------------
@@ -709,7 +764,7 @@ public:
 };
 
 EventNotifierAction::EventNotifierAction(QObject *parent, const char *name)
-: QAction(parent, name)
+: IconAction(parent, name)
 {
 	d = new Private;
 	setMenuText(tr("<Event notifier>"));
@@ -742,7 +797,7 @@ bool EventNotifierAction::addTo(QWidget *w)
 
 void EventNotifierAction::setText(const QString &t)
 {
-	QAction::setText("<nobr>" + t + "</nobr>");
+	IconAction::setText("<nobr>" + t + "</nobr>");
 
 	QPtrListIterator<MLabel> it ( d->labels );
 	for ( ; it.current(); ++it) {
@@ -796,4 +851,22 @@ void EventNotifierAction::show()
 	}
 }
 
+IconAction *EventNotifierAction::copy() const
+{
+	EventNotifierAction *act = new EventNotifierAction( 0 );
+
+	*act = *this;
+
+	return act;
+}
+
+EventNotifierAction &EventNotifierAction::operator=( const EventNotifierAction &from )
+{
+	*( (IconAction *)this ) = from;
+
+	d->hide = from.d->hide;
+
+	return *this;
+}
+
 #include "mainwin_p.moc"
