@@ -148,8 +148,9 @@ QWidget *OptionsTabAppearance::widget()
 	QWhatsThis::add(d->pb_cAnimBack,
 		tr("Specifies the background animation color for nicks."));
 
-	QWhatsThis::add(d->ck_newHeadings,
-		tr("Toggles on new, smaller group headings."));
+	// Avatars
+	//QWhatsThis::add(d->ck_avatarsChatdlg,
+	//	tr("Toggles displaying of avatars in the chat dialog"));
 
 	return w;
 }
@@ -159,8 +160,8 @@ void OptionsTabAppearance::applyOptions(Options *opt)
 	if ( !w )
 		return;
 
-	OptAppearanceUI *d = (OptAppearanceUI *)w;
-	opt->clNewHeadings = d->ck_newHeadings->isChecked();
+	//OptAppearanceUI *d = (OptAppearanceUI *)w;
+	//opt->avatarsChatdlgEnabled = d->ck_avatarsChatdlg->isChecked(); // Avatars
 
 	int n;
 	for (n = 0; n < 4; ++n)
@@ -175,8 +176,8 @@ void OptionsTabAppearance::restoreOptions(const Options *opt)
 	if ( !w )
 		return;
 
-	OptAppearanceUI *d = (OptAppearanceUI *)w;
-	d->ck_newHeadings->setChecked( opt->clNewHeadings );
+	//OptAppearanceUI *d = (OptAppearanceUI *)w;
+	//d->ck_avatarsChatdlg->setChecked( opt->avatarsChatdlgEnabled ); // Avatars
 
 	int n;
 	for (n = 0; n < 4; ++n)
