@@ -5,6 +5,8 @@
 #include "iconwidget.h"
 #include "mainwin.h"
 #include "psitoolbar.h"
+#include "iconaction.h"
+#include "psiactionlist.h"
 
 #include "opt_lookfeel_toolbars.h"
 #include "ui_positiontoolbar.h"
@@ -129,9 +131,39 @@ bool PositionToolbarDlg::eventFilter(QObject *watched, QEvent *e)
 // ToolbarDlg
 //----------------------------------------------------------------------------
 
+class ToolbarDlg::Private
+{
+public:
+	struct ToolbarItem {
+		QString group;
+		int index;
+		ToolbarItem() {
+			group = "";
+			index = -1;
+		}
+		ToolbarItem( QString _w, int _i ) {
+			group = _w;
+			index = _i;
+		}
+	};
+
+	QMap<int, ToolbarItem> toolbars;
+
+	PsiActionList::ActionsType class2id( QString name ) {
+		int ret = (int)PsiActionList::Actions_Common;
+
+		if ( name == "mainWin" )
+			ret |= (int)PsiActionList::Actions_MainWin;
+
+		return (PsiActionList::ActionsType)ret;
+	}
+};
+
 ToolbarDlg::ToolbarDlg(PsiCon *_psi, QWidget *parent, const char *name)
 : QDialog(parent, name, false, WDestructiveClose)
 {
+	p = new Private();
+
 	psi  = _psi;
 	psi->dialogRegister(this);
 	noDirty = false;
@@ -218,22 +250,21 @@ ToolbarDlg::~ToolbarDlg()
 {
 	psi->dialogUnregister(this);
 	delete opt;
+	delete p;
 }
 
-void ToolbarDlg::setCurrentToolbar(QToolBar *t)
+void ToolbarDlg::setCurrentToolbar(PsiToolBar *t)
 {
-	int count = 0;
-	MainWin *mainWin = (MainWin *)psi->mainWin();
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
 
 	if ( pb_apply->isEnabled() )
 		return;
 
-	QPtrListIterator<PsiToolBar> it( mainWin->toolbars );
-	for ( ; it.current(); ++it, count++) {
-		if ( it.current() == t ) {
-			d->cb_toolbars->setCurrentItem(count);
-			toolbarSelectionChanged(count);
+	QMap<int, Private::ToolbarItem>::Iterator it = p->toolbars.begin();
+	for ( ; it != p->toolbars.end(); ++it ) {
+		if ( it.data().group == t->group() && it.data().index == t->groupIndex() ) {
+			d->cb_toolbars->setCurrentItem( it.key() );
+			toolbarSelectionChanged( it.key() );
 			break;
 		}
 	}
@@ -250,13 +281,35 @@ void ToolbarDlg::applyOptions(Options *o)
 		//if ( toolbarPositionInProgress && posTbDlg->n() == (int)i )
 		//	continue;
 
-		mainWin->getLocation ( mainWin->toolbars.at(i), opt->toolbars[i].dock, opt->toolbars[i].index, opt->toolbars[i].nl, opt->toolbars[i].extraOffset );
+		Options::ToolbarPrefs &tbPref = opt->toolbars["mainWin"][i];
+		mainWin->getLocation ( mainWin->toolbars.at(i), tbPref.dock, tbPref.index, tbPref.nl, tbPref.extraOffset );
 	}
 
 	// apply options
 	o->toolbars = opt->toolbars;
 }
 
+void ToolbarDlg::rebuildToolbarList()
+{
+	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
+
+	d->cb_toolbars->clear();
+	p->toolbars.clear();
+
+	QMap< QString, QValueList<Options::ToolbarPrefs> >::Iterator it = opt->toolbars.begin();
+	for ( ; it != opt->toolbars.end(); ++it ) {
+		if ( p->toolbars.count() )
+			d->cb_toolbars->insertItem( "---" ); // TODO: think of better separator
+
+		int i = 0;
+		QValueList<Options::ToolbarPrefs>::Iterator it2 = it.data().begin();
+		for ( ; it2 != it.data().end(); ++it2, ++i ) {
+			d->cb_toolbars->insertItem( (*it2).name );
+			p->toolbars[d->cb_toolbars->count()-1] = Private::ToolbarItem( it.key(), i );
+		}
+	}
+}
+
 void ToolbarDlg::restoreOptions(const Options *o)
 {
 	if ( !w )
@@ -265,24 +318,7 @@ void ToolbarDlg::restoreOptions(const Options *o)
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
 	opt->toolbars = o->toolbars;
 
-	d->cb_toolbars->clear();
-
-	uint i = 0;
-	QValueList<Options::ToolbarPrefs>::Iterator it = opt->toolbars.begin();
-	for ( ; it != opt->toolbars.end(); ++it, i++) {
-		d->cb_toolbars->insertItem( (*it).name );
-	}
-
-	MainWin *mainWin = (MainWin *)psi->mainWin();
-	QStringList::Iterator it2 = mainWin->actionList.begin();
-	for ( ; it2 != mainWin->actionList.end(); ++it2) {
-		if ( *it2 != "---" )
-			addToolbarAction(d->lv_availActions, *it2);
-		else {
-			QListViewItem *item = new QListViewItem(d->lv_availActions, d->lv_availActions->lastItem());
-			item->setText(0, "---");
-		}
-	}
+	rebuildToolbarList();
 
 	if(d->cb_toolbars->count() > 0) {
 		d->cb_toolbars->setCurrentItem( 0 );
@@ -314,39 +350,48 @@ void ToolbarDlg::toolbarAdd()
 
 	tb.dirty = true;
 
-	opt->toolbars.append(tb);
+	opt->toolbars["mainWin"].append(tb);
 
+	rebuildToolbarList();
+/*	TODO
 	d->cb_toolbars->insertItem( tb.name );
 	d->cb_toolbars->setCurrentItem( d->cb_toolbars->count() - 1 );
 	toolbarSelectionChanged( d->cb_toolbars->count() - 1 );
 
 	d->le_toolbarName->setFocus();
+*/
 }
 
 void ToolbarDlg::toolbarDelete()
 {
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
-	int n = d->cb_toolbars->currentItem(); //item->text(1).toInt();
+	int n = d->cb_toolbars->currentItem();
+
+	Options::ToolbarPrefs tb = opt->toolbars[p->toolbars[n].group][p->toolbars[n].index];
+	PsiToolBar *toolBar = psi->findToolBar( p->toolbars[n].group, p->toolbars[n].index );
+	if ( !toolBar )
+		return;
 
 	// delete current toolbar
 	{
-		QValueList<Options::ToolbarPrefs>::Iterator it = opt->toolbars.begin();
-		for (int i = 0; i < n; i++)
+		int idx = p->toolbars[n].index;
+		QValueList<Options::ToolbarPrefs>::Iterator it = opt->toolbars[p->toolbars[n].group].begin();
+		for (int i = 0; i < idx; i++)
 			++it;
 
 		noDirty = true;
-		opt->toolbars.remove(it);
+		opt->toolbars[p->toolbars[n].group].remove(it);
 
-		d->cb_toolbars->removeItem(n);
+		rebuildToolbarList();
 		noDirty = false;
-		toolbarSelectionChanged( d->cb_toolbars->currentItem() );
+		toolbarSelectionChanged( n - 1 ); // TODO FIXME
 	}
 }
 
-void ToolbarDlg::addToolbarAction(QListView *parent, QString name)
+void ToolbarDlg::addToolbarAction(QListView *parent, QString name, int toolbarId)
 {
-	MainWin *mainWin = (MainWin *)psi->mainWin();
-	const QAction *action = mainWin->actions[name];
+	ActionList actions = psi->actionList()->suitableActions( (PsiActionList::ActionsType)toolbarId );
+	const QAction *action = (QAction *)actions.action( name );
 	if ( !action )
 		return;
 
@@ -370,22 +415,30 @@ void ToolbarDlg::toolbarSelectionChanged(int item)
 	if ( noDirty )
 		return;
 
+	int n = item;
+	Options::ToolbarPrefs tb = opt->toolbars[p->toolbars[n].group][p->toolbars[n].index];
+	PsiToolBar *toolBar = psi->findToolBar( p->toolbars[n].group, p->toolbars[n].index );
+
+	bool customizeable = toolBar ? toolBar->isCustomizeable() : false;
+	bool moveable      = toolBar ? toolBar->isMoveable() : false;
+
 	LookFeelToolbarsUI *d = (LookFeelToolbarsUI *)w;
 	bool enable = (item == -1) ? false: true;
 	d->le_toolbarName->setEnabled( enable );
-	d->pb_toolbarPosition->setEnabled( enable );
+	d->pb_toolbarPosition->setEnabled( enable && moveable );
 	d->ck_toolbarOn->setEnabled( enable );
-	d->ck_toolbarLocked->setEnabled( enable );
-	d->ck_toolbarStretch->setEnabled( enable );
-	d->lv_selectedActions->setEnabled( enable );
-	d->lv_availActions->setEnabled( enable );
-	d->tb_up->setEnabled( enable );
-	d->tb_down->setEnabled( enable );
-	d->tb_left->setEnabled( enable );
-	d->tb_right->setEnabled( enable );
-	d->pb_deleteToolbar->setEnabled( enable );
+	d->ck_toolbarLocked->setEnabled( enable && moveable );
+	d->ck_toolbarStretch->setEnabled( enable && moveable );
+	d->lv_selectedActions->setEnabled( enable && customizeable );
+	d->lv_availActions->setEnabled( enable && customizeable );
+	d->tb_up->setEnabled( enable && customizeable );
+	d->tb_down->setEnabled( enable && customizeable );
+	d->tb_left->setEnabled( enable && customizeable );
+	d->tb_right->setEnabled( enable && customizeable );
+	d->pb_deleteToolbar->setEnabled( enable && p->toolbars[n].group == "mainWin" );
 	d->cb_toolbars->setEnabled( enable );
 
+	d->lv_availActions->clear();
 	d->lv_selectedActions->clear();
 
 	if ( !enable )
@@ -393,17 +446,45 @@ void ToolbarDlg::toolbarSelectionChanged(int item)
 
 	noDirty = true;
 
-	int n = item;
-	Options::ToolbarPrefs tb = opt->toolbars[n];
-
 	d->le_toolbarName->setText( tb.name );
 	d->ck_toolbarOn->setChecked( tb.on );
-	d->ck_toolbarLocked->setChecked( tb.locked );
+	d->ck_toolbarLocked->setChecked( tb.locked || !moveable );
 	d->ck_toolbarStretch->setChecked( tb.stretchable );
 
+	{
+		// Fill the ListView with toolbar-specific actions
+		QPtrList<ActionList> actionLists = psi->actionList()->actionLists( p->class2id( p->toolbars[n].group ) );
+		QPtrListIterator<ActionList> it ( actionLists );
+		ActionList *actionList;
+		QListView *lv = d->lv_availActions;
+		QListViewItem *lastRoot = 0;
+		for ( ; (actionList = it.current()); ++it ) {
+			QListViewItem *root = new QListViewItem(lv, lastRoot);
+			lastRoot = root;
+			root->setText( 0, actionList->name() );
+			root->setOpen( true );
+
+			QListViewItem *last = 0;
+			QStringList actionNames = actionList->actions();
+			QStringList::Iterator it2 = actionNames.begin();
+			for ( ; it2 != actionNames.end(); ++it2 ) {
+				IconAction *action = actionList->action( *it2 );
+		        	QListViewItem *item = new QListViewItem( root, last );
+				last = item;
+
+				QString n = actionName((QAction *)action);
+				if ( !action->whatsThis().isEmpty() )
+					n += " - " + action->whatsThis();
+				item->setText(0, n);
+				item->setText(1, action->name());
+				item->setPixmap(0, action->iconSet().pixmap());
+			}
+		}
+	}
+
 	QStringList::Iterator it = tb.keys.begin();
 	for ( ; it != tb.keys.end(); ++it) {
-		addToolbarAction(d->lv_selectedActions, *it);
+		addToolbarAction(d->lv_selectedActions, *it, p->class2id( p->toolbars[n].group ) );
 	}
 	updateArrows();
 
@@ -425,8 +506,8 @@ void ToolbarDlg::rebuildToolbarKeys()
 		keys << item->text(1);
         }
 
-	opt->toolbars[n].keys  = keys;
-	opt->toolbars[n].dirty = true;
+	opt->toolbars["mainWin"][n].keys  = keys;
+	opt->toolbars["mainWin"][n].dirty = true;
 	emit dataChanged();
 }
 
@@ -467,7 +548,7 @@ void ToolbarDlg::toolbarNameChanged()
 
 	int n = d->cb_toolbars->currentItem();
 	d->cb_toolbars->changeItem(d->le_toolbarName->text(), n);
-	opt->toolbars[n].name = d->le_toolbarName->text();
+	opt->toolbars["mainWin"][n].name = d->le_toolbarName->text();
 
 	emit dataChanged();
 }
@@ -505,7 +586,7 @@ void ToolbarDlg::toolbarAddAction()
 	if ( !item || item->text(1).isEmpty() )
 		return;
 
-	addToolbarAction(d->lv_selectedActions, item->text(1));
+	addToolbarAction(d->lv_selectedActions, item->text(1), p->class2id( p->toolbars[d->cb_toolbars->currentItem()].group ) );
 	rebuildToolbarKeys();
 	updateArrows();
 }
@@ -536,11 +617,11 @@ void ToolbarDlg::toolbarDataChanged()
 		return;
 	int n = d->cb_toolbars->currentItem();
 
-	opt->toolbars[n].dirty = true;
-	opt->toolbars[n].name = d->le_toolbarName->text();
-	opt->toolbars[n].on = d->ck_toolbarOn->isChecked();
-	opt->toolbars[n].locked = d->ck_toolbarLocked->isChecked();
-	opt->toolbars[n].stretchable = d->ck_toolbarStretch->isChecked();
+	opt->toolbars["mainWin"][n].dirty = true;
+	opt->toolbars["mainWin"][n].name = d->le_toolbarName->text();
+	opt->toolbars["mainWin"][n].on = d->ck_toolbarOn->isChecked();
+	opt->toolbars["mainWin"][n].locked = d->ck_toolbarLocked->isChecked();
+	opt->toolbars["mainWin"][n].stretchable = d->ck_toolbarStretch->isChecked();
 
 	emit dataChanged();
 }
@@ -567,7 +648,7 @@ void ToolbarDlg::toolbarPosition()
 		return;
 	int n = d->cb_toolbars->currentItem();
 
-	PositionToolbarDlg *posTbDlg = new PositionToolbarDlg(this, &opt->toolbars[n], n);
+	PositionToolbarDlg *posTbDlg = new PositionToolbarDlg(this, &opt->toolbars["mainWin"][n], n);
 	connect(posTbDlg, SIGNAL(applyPressed()), SLOT(toolbarPositionApply()));
 
 	posTbDlg->exec();
@@ -586,12 +667,14 @@ void ToolbarDlg::toolbarPositionApply()
 void ToolbarDlg::doApply()
 {
 	int count = 0;
-	QValueList<Options::ToolbarPrefs>::Iterator it = opt->toolbars.begin();
-	for ( ; it != opt->toolbars.end(); ++it) {
+	QValueList<Options::ToolbarPrefs>::Iterator it = opt->toolbars["mainWin"].begin();
+	for ( ; it != opt->toolbars["mainWin"].end(); ++it) {
 		if ( (*it).on )
 			count++;
 	}
 
+	// Undar MacOS, we have the toolbar in a menu anyway
+#ifndef Q_WS_MAC
 	if ( !count ) {
 		QMessageBox::warning(this, tr("Warning"),
 				     tr("You can not disable <i>all</i> toolbars. If you do so, you will be unable to enable them back, when you'll change your mind.\n"
@@ -600,6 +683,7 @@ void ToolbarDlg::doApply()
 				     tr("I understand"));
 		return;
 	}
+#endif
 
 	applyOptions(opt);
 	option.toolbars = opt->toolbars;
