@@ -4,7 +4,7 @@
 #include <qdialog.h>
 
 class PsiCon;
-class QToolBar;
+class PsiToolBar;
 struct Options;
 class QAction;
 class IconButton;
@@ -18,7 +18,7 @@ public:
 	ToolbarDlg(PsiCon *, QWidget *parent = 0, const char *name = 0);
 	~ToolbarDlg();
 
-	void setCurrentToolbar(QToolBar *);
+	void setCurrentToolbar(PsiToolBar *);
 
 signals:
 	void applyOptions(const Options &);
@@ -26,7 +26,7 @@ signals:
 private slots:
 	void toolbarAdd();
 	void toolbarDelete();
-	void addToolbarAction(QListView *, QString name);
+	void addToolbarAction(QListView *, QString name, int toolbarId);
 	void addToolbarAction(QListView *, const QAction *action, QString name);
 	void toolbarSelectionChanged(int);
 	void rebuildToolbarKeys();
@@ -48,6 +48,7 @@ private slots:
 
 	void applyOptions(Options *opt);
 	void restoreOptions(const Options *opt);
+	void rebuildToolbarList();
 
 signals:
 	void dataChanged();
@@ -60,6 +61,9 @@ private:
 	PsiCon *psi;
 	bool noDirty;
 	IconButton *pb_apply;
+
+	class Private;
+	Private *p;
 };
 
 #endif
