@@ -43,7 +43,7 @@ public:
 	QString jid, pass, host, resource;
 	int port, priority;
 	bool opt_enabled, opt_ssl, opt_pass, opt_host, opt_auto, opt_keepAlive, opt_plain, opt_log, opt_reconn, opt_ignoreSSLWarnings;
-	bool tog_offline, tog_away, tog_agents, tog_hidden;
+	bool tog_offline, tog_away, tog_agents, tog_hidden, tog_self;
 
 	int proxy_index;
 	int proxy_type, proxy_port;
