@@ -49,6 +49,7 @@ void UserAccount::reset()
 	tog_away = TRUE;
 	tog_hidden = TRUE;
 	tog_agents = TRUE;
+	tog_self = FALSE;
 	jid = "";
 	pass = "";
 	opt_pass = FALSE;
@@ -95,6 +96,7 @@ QDomElement UserAccount::toXml(QDomDocument &doc, const QString &tagName)
 	setBoolAttribute(a, "showAway", tog_away);
 	setBoolAttribute(a, "showHidden", tog_hidden);
 	setBoolAttribute(a, "showAgents", tog_agents);
+	setBoolAttribute(a, "showSelf", tog_self);
 	setBoolAttribute(a, "keepAlive", opt_keepAlive);
 	setBoolAttribute(a, "plain", opt_plain);
 	setBoolAttribute(a, "log", opt_log);
@@ -204,6 +206,7 @@ void UserAccount::fromXml(const QDomElement &a)
 	readBoolAttribute(a, "showAway", &tog_away);
 	readBoolAttribute(a, "showHidden", &tog_hidden);
 	readBoolAttribute(a, "showAgents", &tog_agents);
+	readBoolAttribute(a, "showSelf", &tog_self);
 	readBoolAttribute(a, "keepAlive", &opt_keepAlive);
 	readBoolAttribute(a, "plain", &opt_plain);
 	readBoolAttribute(a, "log", &opt_log);
@@ -345,16 +348,17 @@ void UserProfile::reset()
 	// prefs
 	prefs.useleft = FALSE;
 	prefs.singleclick = FALSE;
-	prefs.defaultAction = 0;
+	prefs.defaultAction = 1;
 	prefs.delChats = dcHour;
 	prefs.showJoins = TRUE;
 	prefs.browser = 0;
 	prefs.customBrowser = "";
 	prefs.customMailer = "";
 	prefs.alwaysOnTop = win ? TRUE : FALSE;
-	prefs.keepSizes = FALSE;
+	prefs.keepSizes = TRUE;
 	prefs.ignoreHeadline = FALSE;
 	prefs.ignoreNonRoster = FALSE;
+	prefs.excludeGroupChatsFromIgnore = TRUE;
 	prefs.useDock = win ? TRUE: FALSE;
 	prefs.dockDCstyle = win ? TRUE: FALSE;
 	prefs.dockHideMW = FALSE;
@@ -364,11 +368,13 @@ void UserProfile::reset()
 	prefs.popupMsgs = FALSE;
 	prefs.popupChats = FALSE;
 	prefs.popupHeadlines = FALSE;
+	prefs.popupFiles = FALSE;
 	prefs.noAwayPopup = FALSE;
 	prefs.noUnlistedPopup = false;
 	prefs.raise = FALSE;
 	prefs.incomingAs = 0;
 	prefs.askOnline = FALSE;
+	prefs.askOffline = FALSE;
 	prefs.rosterAnim = TRUE;
 	prefs.autoVersion = false;
 	prefs.autoVCardOnLogin = true;
@@ -404,6 +410,17 @@ void UserProfile::reset()
 	prefs.sp.set(QObject::tr("Greece"), QObject::tr("I have gone to a far away place.  I will be back someday!"));
 	prefs.recentStatus.clear();
 
+#ifdef Q_WS_MAC
+	prefs.color[cOnline]    = QColor("#31549F");
+	prefs.color[cListBack]  = QColor("#F0F0F0");
+	prefs.color[cAway]      = QColor("#3A8268");
+	prefs.color[cDND]       = QColor("#800000");
+	prefs.color[cOffline]   = QColor("#000000");
+	prefs.color[cGroupFore] = QColor("#000000");
+	prefs.color[cGroupBack] = QColor("#9CBCFE");
+	prefs.color[cAnimFront] = QColor("#FF0000");
+	prefs.color[cAnimBack] 	= QColor("#000000");
+#else
 	prefs.color[cOnline]    = QColor("#0060C0");
 	prefs.color[cListBack]  = QColor("#C0C0C0");
 	prefs.color[cAway]      = QColor("#008080");
@@ -413,6 +430,7 @@ void UserProfile::reset()
 	prefs.color[cGroupBack] = QColor("#FFFFFF");
 	prefs.color[cAnimFront] = QColor("#FF0000");
 	prefs.color[cAnimBack] 	= QColor("#000000");
+#endif
 
 	prefs.font[fRoster] = QApplication::font().toString();
 	prefs.font[fMessage] = QApplication::font().toString();
@@ -431,7 +449,7 @@ void UserProfile::reset()
 		prefs.smallFontSize = minimumFontSize;
 	FancyLabel::setSmallFontSize( prefs.smallFontSize );
 
-	prefs.clNewHeadings = false;
+	prefs.clNewHeadings = true;
 
 	prefs.player = "play";
 	prefs.noAwaySound = FALSE;
@@ -458,52 +476,63 @@ void UserProfile::reset()
 	prefs.noGCSound = true;
 
 	prefs.pgp = "";
-	prefs.alwaysShowSelf = false;
 
 	// toolbars
 	prefs.toolbars.clear();
-	Options::ToolbarPrefs tb[3];
-	uint i;
-	for (i = 0; i < sizeof(tb)/sizeof(Options::ToolbarPrefs); i++)
+
+	Options::ToolbarPrefs tbDef;
+	tbDef.name = QString::null;
+	tbDef.on = false;
+	tbDef.locked = false;
+	tbDef.stretchable = false;
+	tbDef.keys.clear();
+
+	tbDef.dock = Qt::DockTop;
+	tbDef.nl = true;
+	tbDef.extraOffset = 0;
+
+	tbDef.dirty = true;
+
 	{
-		tb[i].name = QString::null;
-		tb[i].on = false;
-		tb[i].locked = false;
-		tb[i].stretchable = false;
-		tb[i].keys.clear();
+		Options::ToolbarPrefs tb[3];
+		uint i;
+		for (i = 0; i < sizeof(tb)/sizeof(Options::ToolbarPrefs); i++) {
+			tb[i] = tbDef;
+			tb[i].index = i + 1;
+		}
+		
+		bool defaultEnableToolbars;
+#ifdef Q_WS_MAC
+		defaultEnableToolbars = false;
+#else
+		defaultEnableToolbars = true;
+#endif
 
-		tb[i].dock = Qt::DockTop;
-		tb[i].index = i + 1;
-		tb[i].nl = true;
-		tb[i].extraOffset = 0;
+		// Imitate old Psi button layout by default
+		tb[0].name = QObject::tr("Buttons");
+		tb[0].on = defaultEnableToolbars;
+		tb[0].locked = true;
+		tb[0].stretchable = true;
+		tb[0].keys << "button_options" << "button_status";
+		tb[0].dock = Qt::DockBottom;
 
-		tb[i].dirty = true;
+		tb[1].name = QObject::tr("Show contacts");
+		tb[1].on = defaultEnableToolbars;
+		tb[1].locked = true;
+		tb[1].keys << "show_offline" << "show_away" << "show_agents" << "show_self";
+
+		tb[2].name = QObject::tr("Event notifier");
+		tb[2].on = false;
+		tb[2].locked = true;
+		tb[2].stretchable = true;
+		tb[2].keys << "event_notifier";
+		tb[2].dock = Qt::DockBottom;
+		tb[2].index = 0;
+
+		for ( i = 0; i < sizeof(tb)/sizeof(Options::ToolbarPrefs); i++ )
+			prefs.toolbars["mainWin"].append( tb[i] );
 	}
 
-	// Imitate old Psi button layout by default
-	tb[0].name = QObject::tr("Buttons");
-	tb[0].on = true;
-	tb[0].locked = true;
-	tb[0].stretchable = true;
-	tb[0].keys << "button_options" << "button_status";
-	tb[0].dock = Qt::DockBottom;
-	prefs.toolbars.append(tb[0]);
-
-	tb[1].name = QObject::tr("Show contacts");
-	tb[1].on = true;
-	tb[1].locked = true;
-	tb[1].keys << "show_offline" << "show_away" << "show_agents";
-	prefs.toolbars.append(tb[1]);
-
-	tb[2].name = QObject::tr("Event notifier");
-	tb[2].on = false;
-	tb[2].locked = true;
-	tb[2].stretchable = true;
-	tb[2].keys << "event_notifier";
-	tb[2].dock = Qt::DockBottom;
-	tb[2].index = 0;
-	prefs.toolbars.append(tb[2]);
-
 	// groupchat
 	prefs.gcHighlighting = true;
 	prefs.gcHighlights.clear();
@@ -525,6 +554,7 @@ void UserProfile::reset()
 	prefs.ppMessage     = true;
 	prefs.ppChat        = true;
 	prefs.ppHeadline    = true;
+	prefs.ppFile        = true;
 	prefs.ppJidClip     = 25;
 	prefs.ppStatusClip  = -1;
 	prefs.ppTextClip    = 300;
@@ -571,6 +601,9 @@ void UserProfile::reset()
 
 	// auto-auth
 	prefs.autoAuth = false;
+	
+	// message events
+	prefs.messageEvents = true;
 
 	// event priority
 	prefs.eventPriorityHeadline = 0;
@@ -582,6 +615,54 @@ void UserProfile::reset()
 	// data transfer
 	prefs.dtPort = 8010;
 	prefs.dtExternal = "";
+
+	// Avatars
+	//prefs.avatarsEnabled = true;
+	//prefs.avatarsChatdlgEnabled = true;
+	//prefs.avatarsSize = 48;
+}
+
+static Options::ToolbarPrefs loadToolbarData( const QDomElement &e )
+{
+	QDomElement tb_prefs = e;
+	Options::ToolbarPrefs tb;
+
+	readEntry(tb_prefs, "name",		&tb.name);
+	readBoolEntry(tb_prefs, "on",		&tb.on);
+	readBoolEntry(tb_prefs, "locked",	&tb.locked);
+	readBoolEntry(tb_prefs, "stretchable",	&tb.stretchable);
+	xmlToStringList(tb_prefs, "keys",	&tb.keys);
+
+	bool found3 = false;
+	QDomElement tb_position = findSubTag(tb_prefs, "position", &found3);
+	if (found3)
+	{
+		QString dockStr;
+		Qt::Dock dock = Qt::DockTop;
+		readEntry(tb_position, "dock", &dockStr);
+		if (dockStr == "DockTop")
+			dock = Qt::DockTop;
+		else if (dockStr == "DockBottom")
+			dock = Qt::DockBottom;
+		else if (dockStr == "DockLeft")
+			dock = Qt::DockLeft;
+		else if (dockStr == "DockRight")
+			dock = Qt::DockRight;
+		else if (dockStr == "DockMinimized")
+			dock = Qt::DockMinimized;
+		else if (dockStr == "DockTornOff")
+			dock = Qt::DockTornOff;
+		else if (dockStr == "DockUnmanaged")
+			dock = Qt::DockUnmanaged;
+
+		tb.dock = dock;
+
+		readNumEntry(tb_position, "index",		&tb.index);
+		readBoolEntry(tb_position, "nl",		&tb.nl);
+		readNumEntry(tb_position, "extraOffset",	&tb.extraOffset);
+	}
+
+	return tb;
 }
 
 bool UserProfile::toFile(const QString &fname)
@@ -629,7 +710,6 @@ bool UserProfile::toFile(const QString &fname)
 			p_roster.appendChild(textTag(doc, "useleft", prefs.useleft));
 			p_roster.appendChild(textTag(doc, "singleclick", prefs.singleclick));
 			p_roster.appendChild(textTag(doc, "defaultAction", prefs.defaultAction));
-			p_roster.appendChild(textTag(doc, "showself", prefs.alwaysShowSelf));
 			p_roster.appendChild(textTag(doc, "useTransportIconsForContacts", prefs.useTransportIconsForContacts));
 
 			{
@@ -652,6 +732,15 @@ bool UserProfile::toFile(const QString &fname)
 				sorting.appendChild(textTag(doc, "account", name));
 			}
 		}
+
+		// Avatars
+		//{
+			//QDomElement p_avatars = doc.createElement("avatars");
+			//p_general.appendChild(p_avatars);
+			//p_avatars.appendChild(textTag(doc, "use", prefs.avatarsEnabled));
+			//p_avatars.appendChild(textTag(doc, "use_chatdlg", prefs.avatarsChatdlgEnabled));
+			//p_avatars.appendChild(textTag(doc, "size", prefs.avatarsSize));
+		//}
 		{
 			QDomElement p_misc = doc.createElement("misc");
 			p_general.appendChild(p_misc);
@@ -666,6 +755,7 @@ bool UserProfile::toFile(const QString &fname)
 			p_misc.appendChild(textTag(doc, "keepSizes", prefs.keepSizes));
 			p_misc.appendChild(textTag(doc, "ignoreHeadline", prefs.ignoreHeadline));
 			p_misc.appendChild(textTag(doc, "ignoreNonRoster", prefs.ignoreNonRoster));
+			p_misc.appendChild(textTag(doc, "excludeGroupChatIgnore", prefs.excludeGroupChatsFromIgnore));
 			p_misc.appendChild(textTag(doc, "scrollTo", prefs.scrollTo));
 			p_misc.appendChild(textTag(doc, "useEmoticons", prefs.useEmoticons));
 			p_misc.appendChild(textTag(doc, "alertOpenChats", prefs.alertOpenChats));
@@ -677,6 +767,7 @@ bool UserProfile::toFile(const QString &fname)
 			p_misc.appendChild(textTag(doc, "showGroupCounts", prefs.showGroupCounts));
 			p_misc.appendChild(textTag(doc, "jidComplete", prefs.jidComplete));
 			p_misc.appendChild(textTag(doc, "grabUrls", prefs.grabUrls));
+			p_misc.appendChild(textTag(doc, "messageEvents", prefs.messageEvents));
 		}
 		{
 			QDomElement p_dock = doc.createElement("dock");
@@ -710,6 +801,7 @@ bool UserProfile::toFile(const QString &fname)
 
 		p_events.appendChild(textTag(doc, "alertstyle", prefs.alertStyle));
 		p_events.appendChild(textTag(doc, "autoAuth", prefs.autoAuth));
+		
 
 		{
 			QDomElement p_receive = doc.createElement("receive");
@@ -718,6 +810,7 @@ bool UserProfile::toFile(const QString &fname)
 			p_receive.appendChild(textTag(doc, "popupMsgs", prefs.popupMsgs));
 			p_receive.appendChild(textTag(doc, "popupChats", prefs.popupChats));
 			p_receive.appendChild(textTag(doc, "popupHeadlines", prefs.popupHeadlines));
+			p_receive.appendChild(textTag(doc, "popupFiles", prefs.popupFiles));
 			p_receive.appendChild(textTag(doc, "noAwayPopup", prefs.noAwayPopup));
 			p_receive.appendChild(textTag(doc, "noUnlistedPopup", prefs.noUnlistedPopup));
 			p_receive.appendChild(textTag(doc, "raise", prefs.raise));
@@ -745,6 +838,7 @@ bool UserProfile::toFile(const QString &fname)
 			p_pres.appendChild(tag);
 
 			tag.appendChild(textTag(doc, "askOnline", prefs.askOnline));
+			tag.appendChild(textTag(doc, "askOffline", prefs.askOffline));
 			tag.appendChild(textTag(doc, "rosterAnim", prefs.rosterAnim));
 			tag.appendChild(textTag(doc, "autoVersion", prefs.autoVersion));
 			tag.appendChild(textTag(doc, "autoVCardOnLogin", prefs.autoVCardOnLogin));
@@ -847,46 +941,52 @@ bool UserProfile::toFile(const QString &fname)
 		QDomElement p_toolbars = doc.createElement("toolbars");
 		p.appendChild(p_toolbars);
 
-		QValueList<Options::ToolbarPrefs>::Iterator it = prefs.toolbars.begin();
-		for ( ; it != prefs.toolbars.end(); ++it) {
-			Options::ToolbarPrefs tb = *it;
-			if ( tb.keys.size() ||
-			     !tb.name.isEmpty() )
-			//     tb.name != QObject::tr("<unnamed>") ) // don't save empty toolbars
-			{
-				QDomElement tb_prefs = doc.createElement("toolbar");
-				p_toolbars.appendChild(tb_prefs);
+		QMap< QString, QValueList<Options::ToolbarPrefs> >::Iterator itBars = prefs.toolbars.begin();
+		for ( ; itBars != prefs.toolbars.end(); ++itBars ) {
+			QValueList<Options::ToolbarPrefs> toolbars = itBars.data();
+			QDomElement p_bars = doc.createElement( itBars.key() );
+			p_toolbars.appendChild( p_bars );
 
-				tb_prefs.appendChild(textTag(doc, "name",		tb.name));
-				tb_prefs.appendChild(textTag(doc, "on",			tb.on));
-				tb_prefs.appendChild(textTag(doc, "locked",		tb.locked));
-				tb_prefs.appendChild(textTag(doc, "stretchable",	tb.stretchable));
-				tb_prefs.appendChild(stringListToXml(doc, "keys",	tb.keys));
+			QValueList<Options::ToolbarPrefs>::Iterator it = toolbars.begin();
+			for ( ; it != toolbars.end(); ++it) {
+				Options::ToolbarPrefs tb = *it;
+				if ( tb.keys.size() ||
+				     !tb.name.isEmpty() )
+				{
+					QDomElement tb_prefs = doc.createElement("toolbar");
+					p_bars.appendChild(tb_prefs);
 
-				QDomElement tb_position = doc.createElement("position");
-				tb_prefs.appendChild(tb_position);
+					tb_prefs.appendChild(textTag(doc, "name",		tb.name));
+					tb_prefs.appendChild(textTag(doc, "on",			tb.on));
+					tb_prefs.appendChild(textTag(doc, "locked",		tb.locked));
+					tb_prefs.appendChild(textTag(doc, "stretchable",	tb.stretchable));
+					tb_prefs.appendChild(stringListToXml(doc, "keys",	tb.keys));
 
-				QString dockStr;
-				Qt::Dock dock = tb.dock;
-				if (dock == Qt::DockTop)
-					dockStr = "DockTop";
-				else if (dock == Qt::DockBottom)
-					dockStr = "DockBottom";
-				else if (dock == Qt::DockLeft)
-					dockStr = "DockLeft";
-				else if (dock == Qt::DockRight)
-					dockStr = "DockRight";
-				else if (dock == Qt::DockMinimized)
-					dockStr = "DockMinimized";
-				else if (dock == Qt::DockTornOff)
-					dockStr = "DockTornOff";
-				else if (dock == Qt::DockUnmanaged)
-					dockStr = "DockUnmanaged";
+					QDomElement tb_position = doc.createElement("position");
+					tb_prefs.appendChild(tb_position);
 
-				tb_position.appendChild(textTag(doc, "dock",		dockStr));
-				tb_position.appendChild(textTag(doc, "index",		tb.index));
-				tb_position.appendChild(textTag(doc, "nl",		tb.nl));
-				tb_position.appendChild(textTag(doc, "extraOffset",	tb.extraOffset));
+					QString dockStr;
+					Qt::Dock dock = tb.dock;
+					if (dock == Qt::DockTop)
+						dockStr = "DockTop";
+					else if (dock == Qt::DockBottom)
+						dockStr = "DockBottom";
+					else if (dock == Qt::DockLeft)
+						dockStr = "DockLeft";
+					else if (dock == Qt::DockRight)
+						dockStr = "DockRight";
+					else if (dock == Qt::DockMinimized)
+						dockStr = "DockMinimized";
+					else if (dock == Qt::DockTornOff)
+						dockStr = "DockTornOff";
+					else if (dock == Qt::DockUnmanaged)
+						dockStr = "DockUnmanaged";
+
+					tb_position.appendChild(textTag(doc, "dock",		dockStr));
+					tb_position.appendChild(textTag(doc, "index",		tb.index));
+					tb_position.appendChild(textTag(doc, "nl",		tb.nl));
+					tb_position.appendChild(textTag(doc, "extraOffset",	tb.extraOffset));
+				}
 			}
 		}
 	}
@@ -901,6 +1001,7 @@ bool UserProfile::toFile(const QString &fname)
 		pp.appendChild(textTag(doc, "message", prefs.ppMessage));
 		pp.appendChild(textTag(doc, "chat", prefs.ppChat));
 		pp.appendChild(textTag(doc, "headline", prefs.ppHeadline));
+		pp.appendChild(textTag(doc, "file", prefs.ppFile));
 		pp.appendChild(textTag(doc, "jidClip", prefs.ppJidClip));
 		pp.appendChild(textTag(doc, "statusClip", prefs.ppStatusClip));
 		pp.appendChild(textTag(doc, "textClip", prefs.ppTextClip));
@@ -1113,7 +1214,6 @@ bool UserProfile::fromFile(const QString &fname)
 				readBoolEntry(p_roster, "useleft", &prefs.useleft);
 				readBoolEntry(p_roster, "singleclick", &prefs.singleclick);
 				readNumEntry(p_roster, "defaultAction", &prefs.defaultAction);
-				readBoolEntry(p_roster, "showself", &prefs.alwaysShowSelf);
 				readBoolEntry(p_roster, "useTransportIconsForContacts", &prefs.useTransportIconsForContacts);
 
 				QDomElement sorting = findSubTag(p_roster, "sortStyle", &found);
@@ -1140,6 +1240,14 @@ bool UserProfile::fromFile(const QString &fname)
 				}
 			}
 
+			// Avatars
+			//QDomElement avTag = findSubTag(p_general, "avatars", &found);
+			//if(found) {
+			//	readBoolEntry(avTag, "use", &prefs.avatarsEnabled);
+			//	readBoolEntry(avTag, "use_chatdlg", &prefs.avatarsChatdlgEnabled);
+			//	readNumEntry(avTag, "size", &prefs.avatarsSize);
+			//}
+
 			QDomElement tag = findSubTag(p_general, "misc", &found);
 			if(found) {
 				readNumEntry(tag, "delChats", &prefs.delChats);
@@ -1151,6 +1259,7 @@ bool UserProfile::fromFile(const QString &fname)
 				readBoolEntry(tag, "keepSizes", &prefs.keepSizes);
 				readBoolEntry(tag, "ignoreHeadline", &prefs.ignoreHeadline);
 				readBoolEntry(tag, "ignoreNonRoster", &prefs.ignoreNonRoster);
+				readBoolEntry(tag, "excludeGroupChatIgnore", &prefs.excludeGroupChatsFromIgnore);
 				readBoolEntry(tag, "scrollTo", &prefs.scrollTo);
 				readBoolEntry(tag, "useEmoticons", &prefs.useEmoticons);
 				readBoolEntry(tag, "alertOpenChats", &prefs.alertOpenChats);
@@ -1163,6 +1272,7 @@ bool UserProfile::fromFile(const QString &fname)
 				readBoolEntry(tag, "jidComplete", &prefs.jidComplete);
 				readBoolEntry(tag, "grabUrls", &prefs.grabUrls);
 				readBoolEntry(tag, "smallChats", &prefs.smallChats);
+				readBoolEntry(tag, "messageEvents", &prefs.messageEvents);
 			}
 
 			tag = findSubTag(p_general, "dock", &found);
@@ -1198,6 +1308,7 @@ bool UserProfile::fromFile(const QString &fname)
 				readBoolEntry(tag, "popupMsgs", &prefs.popupMsgs);
 				readBoolEntry(tag, "popupChats", &prefs.popupChats);
 				readBoolEntry(tag, "popupHeadlines", &prefs.popupHeadlines);
+				readBoolEntry(tag, "popupFiles", &prefs.popupFiles);
 				readBoolEntry(tag, "noAwayPopup", &prefs.noAwayPopup);
 				readBoolEntry(tag, "noUnlistedPopup", &prefs.noUnlistedPopup);
 				readBoolEntry(tag, "raise", &prefs.raise);
@@ -1221,6 +1332,7 @@ bool UserProfile::fromFile(const QString &fname)
 			QDomElement tag = findSubTag(p_pres, "misc", &found);
 			if(found) {
 				readBoolEntry(tag, "askOnline", &prefs.askOnline);
+				readBoolEntry(tag, "askOffline", &prefs.askOffline);
 				readBoolEntry(tag, "rosterAnim", &prefs.rosterAnim);
 				readBoolEntry(tag, "autoVersion", &prefs.autoVersion);
 				readBoolEntry(tag, "autoVCardOnLogin", &prefs.autoVCardOnLogin);
@@ -1326,53 +1438,59 @@ bool UserProfile::fromFile(const QString &fname)
 
 		QDomElement p_toolbars = findSubTag(p, "toolbars", &found);
 		if (found) {
-			prefs.toolbars.clear();
+			QStringList goodTags;
+			goodTags << "toolbar";
+			goodTags << "mainWin";
+
+			bool mainWinCleared = false;
+			bool oldStyle = true;
 
 			for(QDomNode n = p_toolbars.firstChild(); !n.isNull(); n = n.nextSibling()) {
 				QDomElement e = n.toElement();
 				if( e.isNull() )
 					continue;
 
-				if ( e.tagName().left(7) == "toolbar" ) {
-					QDomElement tb_prefs = e;
-					Options::ToolbarPrefs tb;
+				QString tbGroup;
+				bool isGood = false;
+				QStringList::Iterator it = goodTags.begin();
+				for ( ; it != goodTags.end(); ++it ) {
+					if ( e.tagName().left( (*it).length() ) == *it ) {
+						isGood = true;
 
-					readEntry(tb_prefs, "name",		&tb.name);
-					readBoolEntry(tb_prefs, "on",		&tb.on);
-					readBoolEntry(tb_prefs, "locked",	&tb.locked);
-					readBoolEntry(tb_prefs, "stretchable",	&tb.stretchable);
-					xmlToStringList(tb_prefs, "keys",	&tb.keys);
+						if ( e.tagName().left(7) == "toolbar" )
+							tbGroup = "mainWin";
+						else {
+							tbGroup = *it;
+							oldStyle = false;
+						}
 
-					bool found3 = false;
-					QDomElement tb_position = findSubTag(tb_prefs, "position", &found3);
-					if (found3)
-					{
-						QString dockStr;
-						Qt::Dock dock = Qt::DockTop;
-						readEntry(tb_position, "dock", &dockStr);
-						if (dockStr == "DockTop")
-							dock = Qt::DockTop;
-						else if (dockStr == "DockBottom")
-							dock = Qt::DockBottom;
-						else if (dockStr == "DockLeft")
-							dock = Qt::DockLeft;
-						else if (dockStr == "DockRight")
-							dock = Qt::DockRight;
-						else if (dockStr == "DockMinimized")
-							dock = Qt::DockMinimized;
-						else if (dockStr == "DockTornOff")
-							dock = Qt::DockTornOff;
-						else if (dockStr == "DockUnmanaged")
-							dock = Qt::DockUnmanaged;
+						break;
+					}
+				}
 
-						tb.dock = dock;
-
-						readNumEntry(tb_position, "index",		&tb.index);
-						readBoolEntry(tb_position, "nl",		&tb.nl);
-						readNumEntry(tb_position, "extraOffset",	&tb.extraOffset);
+				if ( isGood ) {
+					if ( tbGroup != "mainWin" || !mainWinCleared ) {
+						prefs.toolbars[tbGroup].clear();
+						if ( tbGroup == "mainWin" )
+							mainWinCleared = true;
 					}
 
-					prefs.toolbars.append(tb);
+					if ( oldStyle ) {
+						Options::ToolbarPrefs tb = loadToolbarData( e );
+						prefs.toolbars[tbGroup].append(tb);
+					}
+					else {
+						for(QDomNode nn = e.firstChild(); !nn.isNull(); nn = nn.nextSibling()) {
+							QDomElement ee = nn.toElement();
+							if( ee.isNull() )
+								continue;
+
+							if ( ee.tagName() == "toolbar" ) {
+								Options::ToolbarPrefs tb = loadToolbarData( ee );
+								prefs.toolbars[tbGroup].append(tb);
+							}
+						}
+					}
 				}
 			}
 
@@ -1380,8 +1498,8 @@ bool UserProfile::fromFile(const QString &fname)
 			if ( progver == "0.9" || progver == "0.9-CVS" ) {
 				// at first, we need to scan the options, to determine, whether event_notifier already available
 				bool found = false;
-				QValueList<Options::ToolbarPrefs>::Iterator it = prefs.toolbars.begin();
-				for ( ; it != prefs.toolbars.end(); ++it) {
+				QValueList<Options::ToolbarPrefs>::Iterator it = prefs.toolbars["mainWin"].begin();
+				for ( ; it != prefs.toolbars["mainWin"].end(); ++it) {
 					QStringList::Iterator it2 = (*it).keys.begin();
 					for ( ; it2 != (*it).keys.end(); ++it2) {
 						if ( *it2 == "event_notifier" ) {
@@ -1400,7 +1518,7 @@ bool UserProfile::fromFile(const QString &fname)
 					tb.keys << "event_notifier";
 					tb.dock  = Qt::DockBottom;
 					tb.index = 0;
-					prefs.toolbars.append(tb);
+					prefs.toolbars["mainWin"].append(tb);
 				}
 			}
 		}
@@ -1423,6 +1541,7 @@ bool UserProfile::fromFile(const QString &fname)
 			readBoolEntry(p_popup, "message", &prefs.ppMessage);
 			readBoolEntry(p_popup, "chat", &prefs.ppChat);
 			readBoolEntry(p_popup, "headline", &prefs.ppHeadline);
+			readBoolEntry(p_popup, "file", &prefs.ppFile);
 			readNumEntry(p_popup,  "jidClip", &prefs.ppJidClip);
 			readNumEntry(p_popup,  "statusClip", &prefs.ppStatusClip);
 			readNumEntry(p_popup,  "textClip", &prefs.ppTextClip);
@@ -1580,7 +1699,6 @@ bool profileNew(const QString &name)
 	}
 
 	p.mkdir("history");
-	p.mkdir("info");
 	p.mkdir("vcard");
 
 	return TRUE;
