@@ -69,6 +69,7 @@
 #include"iconwidget.h"
 #include"base64.h"
 #include"filetransdlg.h"
+//#include"avatars.h"
 
 #include"bsocket.h"
 /*#ifdef Q_WS_WIN
@@ -299,6 +300,9 @@ public:
 	BlockTransportPopupList *blockTransportPopupList;
 	int userCounter;
 
+	// Avatars
+	//AvatarFactory* avatarFactory;
+
 	// pgp related
 	PassphraseDlg *ppdlg;
 	QGuardedPtr<OpenPGP::Request> ppreq;
@@ -351,6 +355,9 @@ PsiAccount::PsiAccount(const UserAccount &acc, PsiCon *parent)
 	d->client = 0;
 	d->cp = 0;
 
+	// Avatars 
+	//d->avatarFactory = new AvatarFactory(this);
+
 	d->blockTransportPopupList = new BlockTransportPopupList();
 
 	d->doPopups = true;
@@ -375,6 +382,9 @@ PsiAccount::PsiAccount(const UserAccount &acc, PsiCon *parent)
 	d->self.setSubscription(Subscription::Both);
 	d->nickFromVCard = false;
 
+	// Avatars
+	//d->self.setAvatarFactory(d->avatarFactory);
+
 	d->messageQueue.setAutoDelete(true);
 	d->ppdlg = 0;
 	d->ppreq = 0;
@@ -448,6 +458,7 @@ PsiAccount::PsiAccount(const UserAccount &acc, PsiCon *parent)
 	connect(d->cp, SIGNAL(actionGroupRemove(const Jid &, const QString &)),SLOT(actionGroupRemove(const Jid &, const QString &)));
 	connect(d->cp, SIGNAL(actionTest(const Jid &)),SLOT(actionTest(const Jid &)));
 	connect(d->cp, SIGNAL(actionSendFile(const Jid &)),SLOT(actionSendFile(const Jid &)));
+	connect(d->cp, SIGNAL(actionSendFiles(const Jid &, const QStringList&)),SLOT(actionSendFiles(const Jid &, const QStringList&)));
 	connect(d->cp, SIGNAL(actionDisco(const Jid &, const QString &)),SLOT(actionDisco(const Jid &, const QString &)));
 	connect(d->cp, SIGNAL(actionInvite(const Jid &, const QString &)),SLOT(actionInvite(const Jid &, const QString &)));
 	connect(d->cp, SIGNAL(actionAssignKey(const Jid &)),SLOT(actionAssignKey(const Jid &)));
@@ -474,7 +485,7 @@ PsiAccount::PsiAccount(const UserAccount &acc, PsiCon *parent)
 	connect(d->psi, SIGNAL(pgpToggled(bool)), SLOT(pgpToggled(bool)));
 	connect(d->psi, SIGNAL(pgpKeysUpdated()), SLOT(pgpKeysUpdated()));
 
-	d->psi->setToggles(d->acc.tog_offline, d->acc.tog_away, d->acc.tog_agents, d->acc.tog_hidden);
+	d->psi->setToggles(d->acc.tog_offline, d->acc.tog_away, d->acc.tog_agents, d->acc.tog_hidden,d->acc.tog_self);
 
 	setEnabled(d->acc.opt_enabled);
 
@@ -568,7 +579,7 @@ const QString & PsiAccount::name() const
 
 const UserAccount & PsiAccount::userAccount() const
 {
-	d->psi->getToggles(&d->acc.tog_offline, &d->acc.tog_away, &d->acc.tog_agents, &d->acc.tog_hidden);
+	d->psi->getToggles(&d->acc.tog_offline, &d->acc.tog_away, &d->acc.tog_agents, &d->acc.tog_hidden,&d->acc.tog_self);
 
 	// save the roster and pgp key bindings
 	d->acc.roster.clear();
@@ -615,6 +626,12 @@ PsiCon *PsiAccount::psi() const
 	return d->psi;
 }
 
+// Avatars
+//AvatarFactory *PsiAccount::avatarFactory() const
+//{
+//	return d->avatarFactory;
+//}
+
 QString PsiAccount::pgpKey() const
 {
 	return d->cur_pgpSecretKeyID;
@@ -663,13 +680,7 @@ void PsiAccount::setUserAccount(const UserAccount &acc)
 	if(!isActive()) {
 		d->jid = j;
 		d->self.setJid(j);
-		// only update 'self' if multiple resources or forced
-		if(option.alwaysShowSelf || d->self.userResourceList().count() > 1) {
-			if(!j.user().isEmpty())
-				d->cp->updateSelf(d->self);
-			else
-				d->cp->removeSelf();
-		}
+		d->cp->updateEntry(d->self);
 	}
 	if(!d->nickFromVCard)
 		setNick(j.user());
@@ -862,7 +873,7 @@ void PsiAccount::login()
 }
 
 // disconnect or stop reconnecting
-void PsiAccount::logout(bool fast)
+void PsiAccount::logout(bool fast, const Status &s)
 {
 	if(!isActive())
 		return;
@@ -872,7 +883,6 @@ void PsiAccount::logout(bool fast)
 
 	if(loggedIn()) {
 		// send logout status
-		Status s("", "Logged out", 0, false);
 		d->client->setPresence(s);
 	}
 
@@ -883,6 +893,13 @@ void PsiAccount::logout(bool fast)
 	v_isActive = false;
 	stateChanged();
 
+	QTimer::singleShot(0, this, SLOT(disconnect()));
+}
+
+// skz note: I had to split logout() because server seem to need some time to store status
+// before stream is closed (weird, I know)
+void PsiAccount::disconnect()
+{
 	// disconnect
 	d->client->close();
 	cleanupStream();
@@ -1203,6 +1220,7 @@ void PsiAccount::client_rosterItemUpdated(const RosterItem &r)
 		// we don't have it at all, so add it
 		u = new UserListItem;
 		u->setRosterItem(r);
+		//u->setAvatarFactory(d->avatarFactory); // Avatars
 		d->userList.append(u);
 	}
 	u->setInList(true);
@@ -1281,7 +1299,7 @@ void PsiAccount::client_resourceAvailable(const Jid &j, const Resource &r)
 			UserResource ur(r);
 			//ur.setSecurityEnabled(true);
 			if(local)
-				ur.setVersionString(PROG_NAME + " " + PROG_VERSION + " / " + getOSName());
+				ur.setClient(PROG_NAME,PROG_VERSION,getOSName());
 			rp = &(*u->userResourceList().append(ur));
 
 			if(notifyOnlineOk && !local) {
@@ -1409,6 +1427,12 @@ void PsiAccount::client_presenceError(const Jid &j, int, const QString &str)
 
 void PsiAccount::client_messageReceived(const Message &_m)
 {
+	//check if it's a server message without a from, and set the from appropriately
+	if (_m.from().isEmpty())
+	{
+		_m.from()=jid().domain();
+	}	
+
 	// if the sender is already in the queue, then queue this message also
 	QPtrListIterator<Message> it(d->messageQueue);
 	for(Message *mi; (mi = it.current()); ++it) {
@@ -1433,7 +1457,7 @@ void PsiAccount::client_messageReceived(const Message &_m)
 void PsiAccount::processIncomingMessage(const Message &_m)
 {
 	// skip empty messages
-	if(_m.body().isEmpty() && _m.urlList().isEmpty() && _m.invite().isEmpty())
+	if(_m.body().isEmpty() && _m.urlList().isEmpty() && _m.invite().isEmpty() && !_m.containsEvents())
 		return;
 
 	// skip headlines?
@@ -1449,8 +1473,8 @@ void PsiAccount::processIncomingMessage(const Message &_m)
 		return;
 	}
 
-	// only toggle if not an invite
-	if(_m.invite().isEmpty())
+	// only toggle if not an invite or body is not empty
+	if(_m.invite().isEmpty() && !_m.body().isEmpty())
 		toggleSecurity(_m.from(), _m.wasEncrypted());
 
 	UserListItem *u = findFirstRelavent(_m.from());
@@ -1468,8 +1492,22 @@ void PsiAccount::processIncomingMessage(const Message &_m)
 	QPtrList<UserListItem> ul = findRelavent(m.from());
 
 	// ignore events from non-roster JIDs?
-	if(ul.isEmpty() && option.ignoreNonRoster)
-		return;
+	if (ul.isEmpty() && option.ignoreNonRoster)
+	{
+		if (option.excludeGroupChatsFromIgnore)
+		{
+			GCMainDlg *w = (GCMainDlg *)dialogFind("GCMainDlg", Jid(_m.from().userHost()));
+			if(!w)
+			{
+				return;
+			}
+				
+		}
+		else
+		{
+			return;
+		}
+	}
 
 	if(ul.isEmpty())
 		j = m.from().userHost();
@@ -1641,7 +1679,7 @@ void PsiAccount::setStatus(const Status &_s)
 	}
 	else {
 		if(isActive())
-			logout();
+			logout(false, s);
 	}
 }
 
@@ -1650,9 +1688,6 @@ void PsiAccount::setStatusDirect(const Status &_s)
 	Status s = _s;
 	s.setPriority(d->acc.priority);
 
-	if(s.status().isEmpty())
-		s.setStatus(status2txt(makeSTATUS(s)));
-
 	//printf("setting status to [%s]\n", s.status().latin1());
 
 	// using pgp?
@@ -1888,14 +1923,6 @@ void PsiAccount::openAddUserDlg()
 	}
 }
 
-void PsiAccount::doManageServices()
-{
-	ServicesDlg *w = new ServicesDlg(d->jid.host(), this);
-	connect(w, SIGNAL(signalRegister(const Jid &)), SLOT(actionRegister(const Jid &)));
-	connect(w, SIGNAL(signalSearch(const Jid &)), SLOT(actionSearch(const Jid &)));
-	w->show();
-}
-
 void PsiAccount::doDisco()
 {
 	actionDisco(d->jid.host(), "");
@@ -2022,7 +2049,7 @@ void PsiAccount::slotClientVersionFinished()
 			if(!found)
 				continue;
 
-			(*rit).setVersionString(j->name() + ' ' + j->version() + " / " + j->os());
+			(*rit).setClient(j->name(),j->version(),j->os());
 
 			cpUpdate(*u);
 			/*if(u->isSelf()) {
@@ -2083,17 +2110,7 @@ void PsiAccount::cpUpdate(const UserListItem &u, const QString &rname, bool from
 {
 	PsiEvent *e = d->eventQueue->peek(u.jid());
 
-	if(u.isSelf()) {
-		if(e || option.alwaysShowSelf || d->self.userResourceList().count() > 1)
-			d->cp->updateSelf(d->self);
-		else
-			d->cp->removeSelf();
-
-		//if(d->cp->self())
-		//	d->cp->updateSelf(u);
-	}
-	else
-		d->cp->updateEntry(u);
+	d->cp->updateEntry(u);
 
 	if(e) {
 		d->cp->setAlert(u.jid(), is->event2icon(e));
@@ -2166,8 +2183,8 @@ ChatDlg *PsiAccount::ensureChatDlg(const Jid &j)
 
 void PsiAccount::changeStatus(int x)
 {
-	if(x == STATUS_OFFLINE) {
-		setStatus(Status("","",0,false));
+	if(x == STATUS_OFFLINE && !option.askOffline) {
+		setStatus(Status("","Logged out",0,false));
 	}
 	else {
 		if(x == STATUS_ONLINE && !option.askOnline) {
@@ -2204,6 +2221,12 @@ void PsiAccount::actionTest(const Jid &j)
 }
 
 void PsiAccount::actionSendFile(const Jid &j)
+{
+	QStringList l;
+	actionSendFiles(j, l);
+}
+
+void PsiAccount::actionSendFiles(const Jid &j, const QStringList& l)
 {
 	Jid j2 = j;
 	if(j.resource().isEmpty()) {
@@ -2212,9 +2235,19 @@ void PsiAccount::actionSendFile(const Jid &j)
 			j2.setResource((*u->userResourceList().priority()).name());
 	}
 
-	//printf("psiaccount: sending file to [%s]\n", j2.full().latin1());
-	FileRequestDlg *w = new FileRequestDlg(j2, d->psi, this);
-	w->show();
+	// Create a dialog for each file in the list. Once the xfer dialog itself
+	// supports multiple files, only the 'else' branch needs to stay.
+	if (!l.isEmpty()) {
+		for (QStringList::ConstIterator f = l.begin(); f != l.end(); ++f ) {
+			QStringList fl(*f);
+			FileRequestDlg *w = new FileRequestDlg(j2, d->psi, this, fl);
+			w->show();
+		}
+	}
+	else {
+		FileRequestDlg *w = new FileRequestDlg(j2, d->psi, this, l);
+		w->show();
+	}
 }
 
 void PsiAccount::actionDefault(const Jid &j)
@@ -2273,6 +2306,7 @@ void PsiAccount::actionSendUrl(const Jid &j)
 
 void PsiAccount::actionRemove(const Jid &j)
 {
+	//avatarFactory()->removeManualAvatar(j); // Avatars
 	dj_remove(j);
 }
 
@@ -2546,8 +2580,8 @@ void PsiAccount::dj_sendMessage(const Message &m, bool log)
 
 	d->client->sendMessage(nm);
 
-	// only toggle if not an invite
-	if(m.invite().isEmpty())
+	// only toggle if not an invite or body is not empty
+	if(m.invite().isEmpty() && !m.body().isEmpty())
 		toggleSecurity(m.to(), m.wasEncrypted());
 
 	// don't log groupchat, private messages, or encrypted messages
@@ -2561,8 +2595,8 @@ void PsiAccount::dj_sendMessage(const Message &m, bool log)
 		}
 	}
 
-	// don't sound when sending groupchat messages
-	if(m.type() != "groupchat")
+	// don't sound when sending groupchat messages or message events
+	if(m.type() != "groupchat" && !m.body().isEmpty())
 		playSound(option.onevent[eSend]);
 
 	// auto close an open messagebox (if non-chat)
@@ -2740,14 +2774,27 @@ void PsiAccount::handleEvent(PsiEvent *e)
 	if(d->acc.opt_log) {
 		if(e->type() == PsiEvent::Message || e->type() == PsiEvent::Auth) {
 			// don't log private messages
-			if(!findGCContact(e->from()))
+			if(!findGCContact(e->from()) && !(e->type() == PsiEvent::Message && ((MessageEvent *)e)->message().body().isEmpty()))
 				logEvent(e->from(), e);
 		}
 	}
+
 	if(e->type() == PsiEvent::Message) {
 		MessageEvent *me = (MessageEvent *)e;
 		const Message &m = me->message();
 
+		// Pass message events to chat window
+		if (m.containsEvents() && m.body().isEmpty()) {
+			if (option.messageEvents) {
+				ChatDlg *c = (ChatDlg *)dialogFind("ChatDlg", e->from());
+				if(!c)
+					c = (ChatDlg *)dialogFind("ChatDlg", e->jid());
+				if (c) 
+					c->incomingMessage(m);
+			}
+			return;
+		}
+
 		// pass chat messages directly to a chat window if possible (and deal with sound)
 		if(m.type() == "chat") {
 			ChatDlg *c = (ChatDlg *)dialogFind("ChatDlg", e->from());
@@ -2803,6 +2850,10 @@ void PsiAccount::handleEvent(PsiEvent *e)
 	}
 	else if(e->type() == PsiEvent::File) {
 		playSound(option.onevent[eIncomingFT]);
+		if (option.ppFile) {
+			doPopup = true;
+			popupType = PsiPopup::AlertFile;
+		}
 	}
 	else {
 		playSound(option.onevent[eSystem]);
@@ -2825,6 +2876,8 @@ void PsiAccount::handleEvent(PsiEvent *e)
 		doPopup = false;
 	if ( popupType == PsiPopup::AlertHeadline && !option.ppHeadline )
 		doPopup = false;
+	if ( popupType == PsiPopup::AlertFile && !option.ppFile )
+		doPopup = false;
 	if ( doPopup && d->doPopups ) {
 
 		Resource r;
@@ -2852,6 +2905,7 @@ void PsiAccount::queueEvent(PsiEvent *e)
 		u = new UserListItem;
 		u->setJid(e->jid());
 		u->setInList(false);
+		// u->setAvatarFactory(d->avatarFactory); // Avatars
 
 		// is it a private groupchat?
 		Jid j = u->jid();
@@ -2911,6 +2965,8 @@ void PsiAccount::queueEvent(PsiEvent *e)
 				doPopup = option.popupHeadlines;
 			else
 				doPopup = option.popupMsgs;
+		} else if (e->type() == PsiEvent::File) {
+			doPopup = option.popupFiles;
 		}
 		else
 			doPopup = option.popupMsgs;
@@ -3492,6 +3548,12 @@ void PsiAccount::pgp_finished(bool b)
 			mwrap.setBody(tr("[This message is encrypted.]"));
 			mwrap.setXEncrypted(enc);
 			mwrap.setWasEncrypted(true);
+			// FIXME: Should be done cleaner, with an extra method in Iris
+			if (m.containsEvent(OfflineEvent)) mwrap.addEvent(OfflineEvent);
+			if (m.containsEvent(DeliveredEvent)) mwrap.addEvent(DeliveredEvent);
+			if (m.containsEvent(DisplayedEvent)) mwrap.addEvent(DisplayedEvent);
+			if (m.containsEvent(ComposingEvent)) mwrap.addEvent(ComposingEvent);
+			if (m.containsEvent(CancelEvent)) mwrap.addEvent(CancelEvent);
 			dj_sendMessage(mwrap);
 		}
 		encryptedMessageSent(x, b);
@@ -3590,10 +3652,7 @@ void PsiAccount::processEncryptedMessageDone()
 
 void PsiAccount::optionsUpdate()
 {
-	if(option.alwaysShowSelf)
-		d->cp->updateSelf(d->self);
-	else if(d->self.userResourceList().count() < 2)
-		d->cp->removeSelf();
+	d->cp->updateEntry(d->self);
 }
 
 QString PsiAccount::resultToString(int result)
@@ -3656,6 +3715,7 @@ void PsiAccount::invokeGCMessage(const Jid &j)
 	u->setInList(false);
 	u->setName(j.resource());
 	u->setPrivate(true);
+	//u->setAvatarFactory(d->avatarFactory); // Avatars
 
 	// make a resource so the contact appears online
 	UserResource ur;
@@ -3680,6 +3740,7 @@ void PsiAccount::invokeGCChat(const Jid &j)
 	u->setInList(false);
 	u->setName(j.resource());
 	u->setPrivate(true);
+	//u->setAvatarFactory(d->avatarFactory); // Avatars
 
 	// make a resource so the contact appears online
 	UserResource ur;
