@@ -56,6 +56,7 @@ class ChatDlg;
 class EDB;
 class QSSLCert;
 class QHostAddress;
+//class AvatarFactory;
 
 class PGPKeyDlg : public PGPKeyUI
 {
@@ -146,11 +147,14 @@ public:
 	EDB *edb() const;
 	PsiCon *psi() const;
 
+	// Avatars
+	//AvatarFactory *avatarFactory() const;
+
 	Status status() const;
 	void setStatusDirect(const Status &);
 	void setStatusActual(const Status &);
 	void login();
-	void logout(bool fast=false);
+	void logout(bool fast=false, const Status &s = Status("", "Logged out", 0, false));
 	bool loggedIn() const;
 	void setNick(const QString &);
 	QString nick() const;
@@ -165,7 +169,6 @@ public:
 	void changeVCard();
 	void changePW();
 	void changeStatus(int);
-	void doManageServices();
 	void doDisco();
 
 	void showXmlConsole();
@@ -259,6 +262,7 @@ public slots:
 	void actionDisco(const Jid &, const QString &);
 	void actionInvite(const Jid &, const QString &);
 	void actionSendFile(const Jid &);
+	void actionSendFiles(const Jid &, const QStringList&);
 	void featureActivated(QString feature, Jid jid, QString node);
 
 	void actionAssignKey(const Jid &);
@@ -296,6 +300,7 @@ private slots:
 	void client_incomingFileTransfer();
 
 	void reconnect();
+	void disconnect();
 	void enableNotifyOnline();
 
 	void slotClientVersionFinished();
