@@ -28,6 +28,7 @@
 #include<qcolor.h>
 #include<qimage.h>
 #include<qstylesheet.h>
+#include<qpixmapcache.h>
 
 #include"s5b.h"
 #include"psiaccount.h"
@@ -56,6 +57,7 @@
 #include"psitoolbar.h"
 #include"filetransfer.h"
 #include"filetransdlg.h"
+#include"psiactionlist.h"
 
 //----------------------------------------------------------------------------
 // PsiIconFactory
@@ -191,13 +193,17 @@ private:
 //----------------------------------------------------------------------------
 // AccountsComboBox
 //----------------------------------------------------------------------------
-AccountsComboBox::AccountsComboBox(PsiCon *_psi, QWidget *parent)
-:QComboBox(parent)
+AccountsComboBox::AccountsComboBox(PsiCon *_psi, QWidget *parent, bool online_only)
+:QComboBox(parent), onlineOnly(online_only)
 {
 	psi = _psi;
+	// TODO: Status changes of accounts should be notified when onlineOnly is true
 	connect(psi, SIGNAL(accountCountChanged()), this, SLOT(updateAccounts()));
 	connect(psi, SIGNAL(destroyed()), SLOT(deleteMe()));
 	connect(this, SIGNAL(activated(int)), this, SLOT(changeAccount()));
+	if (online_only)
+		connect(psi, SIGNAL(accountActivityChanged()), this, SLOT(updateAccounts()));
+
 	pa = 0;
 	setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed ));
 
@@ -223,12 +229,14 @@ void AccountsComboBox::changeAccount()
 	int n = 0;
 	bool found = false;
 	for(PsiAccount *p; (p = it.current()); ++it) {
-		if(i == n) {
-			pa = p;
-			found = true;
-			break;
+		if (!onlineOnly || p->loggedIn()) {
+			if(i == n) {
+				pa = p;
+				found = true;
+				break;
+			}
+			++n;
 		}
-		++n;
 	}
 	if(!found)
 		pa = 0;
@@ -242,23 +250,22 @@ void AccountsComboBox::updateAccounts()
 	PsiAccountListIt it(psi->accountList(TRUE));
 	int n = 0;
 	bool found = false;
+	PsiAccount *firstAccount = 0;
 	for(PsiAccount *p; (p = it.current()); ++it) {
-		insertItem(p->nameWithJid());
-		if(p == pa) {
-			setCurrentItem(n);
-			found = true;
+		if (!onlineOnly || p->loggedIn()) {
+			insertItem(p->nameWithJid());
+			if(p == pa) {
+				setCurrentItem(n);
+				found = true;
+			}
+			if (!firstAccount)
+				firstAccount = p;
+			++n;
 		}
-		++n;
 	}
 	if(!found) {
 		// choose a different account
-		if(psi->accountList(TRUE).isEmpty())
-			pa = 0;
-		else {
-			pa = psi->accountList(TRUE).getFirst();
-			setCurrentItem(0);
-		}
-
+		pa = firstAccount;
 		activated(pa);
 	}
 }
@@ -270,8 +277,9 @@ void AccountsComboBox::deleteMe()
 
 
 //----------------------------------------------------------------------------
-// PsiCon
+// PsiCon::Private
 //----------------------------------------------------------------------------
+
 struct item_dialog
 {
 	QWidget *widget;
@@ -336,8 +344,13 @@ public:
 	IconSelectPopup *iconSelect;
 	QRect mwgeom;
 	FileTransDlg *ftwin;
+	PsiActionList *actionList;
 };
 
+//----------------------------------------------------------------------------
+// PsiCon
+//----------------------------------------------------------------------------
+
 PsiCon::PsiCon()
 :QObject(0)
 {
@@ -359,12 +372,15 @@ PsiCon::PsiCon()
 	d->pgpReady = false;
 	d->s5bServer = 0;
 	d->proxy = 0;
+
+	d->actionList = 0;
 }
 
 PsiCon::~PsiCon()
 {
 	deinit();
 
+	delete d->actionList;
 	delete d->pgp;
 	delete d->edb;
 	delete d;
@@ -376,6 +392,9 @@ bool PsiCon::init()
 	d->pro.reset();
 	d->pro.fromFile(pathToProfileConfig(activeProfile));
 
+	QDir profileDir( pathToProfile( activeProfile ) );
+	profileDir.rmdir( "info" ); // remove unused dir
+
 	d->recentGCList = d->pro.recentGCList;
 	d->recentBrowseList = d->pro.recentBrowseList;
 	d->lastStatusString = d->pro.lastStatusString;
@@ -395,13 +414,15 @@ bool PsiCon::init()
 		//}
 	}
 
+	if ( !d->actionList )
+		d->actionList = new PsiActionList( this );
+
 	d->iconSelect = new IconSelectPopup(0);
 	d->updateIconSelect();
 
 	// setup the main window
 	d->mainwin = new MainWin(option.alwaysOnTop, (option.useDock && option.dockToolMW), this, "psimain");
 	d->mainwin->setUseDock(option.useDock);
-	d->mainwin->setUsingSSL(false);
 
 	connect(d->mainwin, SIGNAL(closeProgram()), SLOT(closeProgram()));
 	connect(d->mainwin, SIGNAL(changeProfile()), SLOT(changeProfile()));
@@ -593,11 +614,13 @@ void PsiCon::link(PsiAccount *pa)
 	d->list.append(pa);
 	if(pa->enabled() && !d->listEnabled.containsRef(pa))
 		d->listEnabled.append(pa);
+	connect(pa, SIGNAL(updatedActivity()), this, SIGNAL(accountActivityChanged()));
 	accountCountChanged();
 }
 
 void PsiCon::unlink(PsiAccount *pa)
 {
+	disconnect(pa, SIGNAL(updatedActivity()), this, SIGNAL(accountActivityChanged()));
 	d->list.remove(pa);
 	d->listEnabled.remove(pa);
 	accountCountChanged();
@@ -716,9 +739,9 @@ void PsiCon::deleteAllDialogs()
 	d->dialogList.clear();
 }
 
-AccountsComboBox *PsiCon::accountsComboBox(QWidget *parent)
+AccountsComboBox *PsiCon::accountsComboBox(QWidget *parent, bool online_only)
 {
-	AccountsComboBox *acb = new AccountsComboBox(this, parent);
+	AccountsComboBox *acb = new AccountsComboBox(this, parent, online_only);
 	return acb;
 }
 
@@ -779,8 +802,8 @@ void PsiCon::enableAccount(PsiAccount *pa, bool e)
 
 void PsiCon::statusMenuChanged(int x)
 {
-	if(x == STATUS_OFFLINE) {
-		setGlobalStatus(Status("","",0,false));
+	if(x == STATUS_OFFLINE && !option.askOffline) {
+		setGlobalStatus(Status("","Logged out",0,false));
 		if(option.useDock == true)
 			d->mainwin->setTrayToolTip(Status("","",0,false));
 	}
@@ -798,13 +821,8 @@ void PsiCon::statusMenuChanged(int x)
 				d->mainwin->setTrayToolTip(s);
 		}
 		else {
-			QString str;
-			if(d->lastStatusString.isEmpty())
-				str = tr("I am away from my computer.  Please leave a message.");
-			else
-				str = d->lastStatusString; // default to old away message
-
-			StatusSetDlg *w = new StatusSetDlg(this, makeStatus(x, str));
+			// Create a dialog with the last status message
+			StatusSetDlg *w = new StatusSetDlg(this, makeStatus(x, d->lastStatusString));
 			connect(w, SIGNAL(set(const Status &)), SLOT(setStatusFromDialog(const Status &)));
 			connect(w, SIGNAL(cancelled()), SLOT(updateMainwinStatus()));
 			if(option.useDock == true)
@@ -822,10 +840,22 @@ void PsiCon::setStatusFromDialog(const Status &s)
 
 void PsiCon::setGlobalStatus(const Status &s)
 {
-	// globally set each account
+	// Check whether all accounts are logged off
+	bool allOffline = true;
+	PsiAccountListIt i(d->listEnabled);
+	PsiAccount *pa;
+	for( ; (pa = i.current()); ++i) {
+		if ( pa->isActive() ) {
+			allOffline = false;
+			break;
+		}
+	}
+
+	// globally set each account which is logged in
 	PsiAccountListIt it(d->listEnabled);
-	for(PsiAccount *pa; (pa = it.current()); ++it)
-		pa->setStatus(s);
+	for( ; (pa = it.current()); ++it)
+		if (allOffline || pa->isActive())
+			pa->setStatus(s);
 }
 
 void PsiCon::pa_updatedActivity()
@@ -884,7 +914,7 @@ void PsiCon::updateMainwinStatus()
 	}
 }
 
-void PsiCon::setToggles(bool tog_offline, bool tog_away, bool tog_agents, bool tog_hidden)
+void PsiCon::setToggles(bool tog_offline, bool tog_away, bool tog_agents, bool tog_hidden, bool tog_self)
 {
 	if(d->listEnabled.count() > 1)
 		return;
@@ -893,14 +923,16 @@ void PsiCon::setToggles(bool tog_offline, bool tog_away, bool tog_agents, bool t
 	d->mainwin->cvlist->setShowAway(tog_away);
 	d->mainwin->cvlist->setShowAgents(tog_agents);
 	d->mainwin->cvlist->setShowHidden(tog_hidden);
+	d->mainwin->cvlist->setShowSelf(tog_self);
 }
 
-void PsiCon::getToggles(bool *tog_offline, bool *tog_away, bool *tog_agents, bool *tog_hidden)
+void PsiCon::getToggles(bool *tog_offline, bool *tog_away, bool *tog_agents, bool *tog_hidden, bool *tog_self)
 {
 	*tog_offline = d->mainwin->cvlist->isShowOffline();
 	*tog_away = d->mainwin->cvlist->isShowAway();
 	*tog_agents = d->mainwin->cvlist->isShowAgents();
 	*tog_hidden = d->mainwin->cvlist->isShowHidden();
+	*tog_self = d->mainwin->cvlist->isShowSelf();
 }
 
 void PsiCon::doOptions()
@@ -952,6 +984,9 @@ void PsiCon::slotApplyOptions(const Options &opt)
 
 		// update icon selector
 		d->updateIconSelect();
+
+		// flush the QPixmapCache
+		QPixmapCache::clear();
 	}
 
 	if ( oldOpt.alertStyle != option.alertStyle )
@@ -1335,4 +1370,30 @@ void PsiCon::s5b_init()
 	}
 }
 
+QPtrList<PsiToolBar> *PsiCon::toolbarList() const
+{
+	QPtrList<PsiToolBar> *list = new QPtrList<PsiToolBar>();
+
+	QPtrListIterator<PsiToolBar> it ( mainWin()->toolbars );
+	for ( ; it.current(); ++it )
+		list->append( it.current() );
+
+	return list;
+}
+
+PsiToolBar *PsiCon::findToolBar(QString group, int index)
+{
+	PsiToolBar *toolBar = 0;
+
+	if ( group == "mainWin" )
+		toolBar = mainWin()->toolbars.at(index);
+
+	return toolBar;
+}
+
+PsiActionList *PsiCon::actionList() const
+{
+	return d->actionList;
+}
+
 #include "psicon.moc"
