@@ -24,6 +24,7 @@
 #include"openpgp.h"
 #include"common.h"
 #include"contactview.h"
+//#include"avatars.h"
 
 using namespace XMPP;
 
@@ -65,9 +66,29 @@ const QString & UserResource::versionString() const
 	return v_ver;
 }
 
-void UserResource::setVersionString(const QString &s)
+const QString & UserResource::clientName() const
 {
-	v_ver = s;
+	return v_clientName;
+}
+
+const QString & UserResource::clientVersion() const
+{
+	return v_clientVersion;
+}
+
+const QString & UserResource::clientOS() const
+{
+	return v_clientOS;
+}
+
+void UserResource::setClient(const QString &name, const QString& version, const QString& os)
+{
+	v_clientName = name;
+	v_clientVersion = version;
+	v_clientOS = os;
+	v_ver = v_clientName + " " + v_clientVersion;
+	if ( !v_clientOS.isEmpty() )
+		v_ver += " / " + v_clientOS;
 }
 
 const QString & UserResource::publicKeyID() const
@@ -197,6 +218,7 @@ UserListItem::UserListItem(bool self)
 	v_self = self;
 	v_private = false;
 	lastmsgtype = -1;
+	v_avatarFactory = 0;
 }
 
 UserListItem::~UserListItem()
@@ -311,7 +333,35 @@ UserResourceList::ConstIterator UserListItem::priority() const
 
 QString UserListItem::makeTip(bool trim, bool doLinkify) const
 {
-	QString str = QString("<qt>");
+	return "<qt>" + makeBareTip(trim,doLinkify) + "</qt>";
+}
+
+QString UserListItem::makeBareTip(bool trim, bool doLinkify) const
+{
+	QString str;
+
+	// Avatars
+	//str += "<table cellspacing=\"0\"><tr>";
+	//if (option.avatarsEnabled) {
+	//	QString client;
+	//	QString res;
+	//	if (!userResourceList().isEmpty()) {
+	//		client = (*userResourceList().priority()).clientName();
+	//		res = (*userResourceList().priority()).name();
+	//	}
+	//	if (avatarFactory()) {
+	//		QPixmap p = avatarFactory()->getAvatar(jid().withResource(res),client);
+	//		if (!p.isNull()) {
+	//			str += "<td align=\"center\">";
+	//			QMimeSourceFactory::defaultFactory()->setPixmap("avatar.png",p);
+	//			str += "<img src=\"avatar.png\">";
+	//			str += "</td>";
+	//		}
+	//	}
+	//}
+
+	//str += "<td>";
+
 	QString nick = jidnick(jid().full(), name());
 	str += QString("<nobr>%1</nobr>").arg(expandEntities(nick));
 	if(jid().full() != nick)
@@ -446,7 +496,9 @@ QString UserListItem::makeTip(bool trim, bool doLinkify) const
 		}
 	}
 
-	str += "</qt>";
+	// Avatars
+	//str += "</td>";
+	//str += "</tr></table>";
 
 	return str;
 }
@@ -498,6 +550,17 @@ void UserListItem::setPublicKeyID(const QString &k)
 	v_keyID = k;
 }
 
+AvatarFactory* UserListItem::avatarFactory() const
+{
+	return v_avatarFactory;
+}
+
+void UserListItem::setAvatarFactory(AvatarFactory* av)
+{
+	v_avatarFactory = av;
+}
+
+
 
 //----------------------------------------------------------------------------
 // UserList
