@@ -49,6 +49,9 @@ private:
 public slots:
 	void taskFinished();
 
+signals:
+	void vcardChanged(const Jid&);
+
 public:
 	VCardFactoryPrivate();
 	~VCardFactoryPrivate();
@@ -118,6 +121,8 @@ void VCardFactoryPrivate::taskFinished()
 		QDomDocument doc;
 		doc.appendChild( vcard->toXml ( &doc ) );
 		out << doc.toString(4);
+
+		emit vcardChanged(j);
 	}
 }
 
@@ -172,6 +177,13 @@ JT_VCard *VCardFactory::getVCard(const Jid &jid, Task *rootTask, const QObject *
 	task->go(true);
 	return task;
 }
+	
+void VCardFactory::registerVCardChanged(const QObject* obj, const char* slot)
+{
+	if ( !vcardFactoryPrivate )
+		vcardFactoryPrivate = new VCardFactoryPrivate;
+	QObject::connect(vcardFactoryPrivate,SIGNAL(vcardChanged(const Jid&)),obj,slot);
+}
 
 #include"vcardfactory.moc"
 
